"""This module creates statistics in yearly bins for timeseries"""

#pylint: disable=line-too-long
#pylint: disable=trailing-whitespace
#pylint: disable=invalid-name

import pandas as pd
from netCDF4 import num2date #pylint: disable=no-name-in-module
import numpy as np
import matplotlib.pyplot as plt
import seaborn as sn

from SuPerSim.mytime import list_tokens_year
from SuPerSim.constants import colorcycle, units
from SuPerSim.seasonal import stats_all_years_simulations_to_single_year

def data_box_yearly_stat(name_series, time_file, file_to_plot, year_bkg_end, year_trans_end):
    """ Returns dataframe of daily data
    
    Parameters
    ----------
    name_series : str
        Name of the quantity to plot, has to be one of 'GST', 'Air temperature', 'Precipitation', 'SWE', 'Water production'
    time_file : netCDF4._netCDF4.Variable
        File where the time index of each datapoint is stored        
    file_to_plot : list
        Mean time series (temp_ground_mean, mean_air_temp, mean_prec, swe_mean, tot_water_prod)
    year_bkg_end : int
        Background period is BEFORE the start of the year corresponding to the variable, i.e. all time stamps before Jan 1st year_bkg_end
    year_trans_end : int
        Same for transient period

    Returns
    -------
    df_data : pandas.core.frame.DataFrame
        Panda dataframe with columns ['name_series' (e.g. 'Precipitation), 'Year']
    """

    list_dates = list_tokens_year(time_file, year_bkg_end, year_trans_end)[0]
    
    a = []
    for i in list_dates.keys():
        if i < year_trans_end:
            a = a + [i]*len(list_dates[i])

    if name_series in ['Precipitation', 'Water production']:
        file_to_plot = [i*86400 for i in file_to_plot]

    df_data = pd.DataFrame(file_to_plot[:len(a)], columns=[name_series], index=a)
    df_data['Year'] = a

    return df_data

def plot_box_yearly_stat(df_data, year_bkg_end, year_trans_end):
    """ Box plot of yearly statistics of the timeseries, together with mean for the background and transient periods
    
    Parameters
    ----------
    df_data : pandas.core.frame.DataFrame
        Panda dataframe with columns ['name_series' (e.g. 'Precipitation), 'Year']
    year_bkg_end : int
        Background period is BEFORE the start of the year corresponding to the variable, i.e. all time stamps before Jan 1st year_bkg_end
    year_trans_end : int
        Same for transient period

    Returns
    -------
    fig : Figure
        Box plot of yearly statistics of the timeseries, together with mean for the background and transient periods
    """

    fig, ax = plt.subplots()

    list_dates = sorted(np.unique(df_data['Year']))
    year_start = list_dates[0]
    name_series = df_data.columns[0]
    mean = [np.mean(df_data[df_data['Year']<year_bkg_end][name_series]), np.mean(df_data[(df_data['Year']>=year_bkg_end) & (df_data['Year']<year_trans_end)][name_series])]
    formatted_mean = [f"{i:.2f}" for i in mean]

    meanpointprops = dict(marker='D', markeredgecolor='black', markerfacecolor='firebrick')
    sn.boxplot(x='Year', y=name_series, data=df_data, showmeans=True, showfliers=False, meanprops=meanpointprops, color='grey', linecolor='black')

    ax.hlines(mean[0], 0 - 1/2, year_bkg_end - year_start - 1/2, linewidth=2, color=colorcycle[0],
              label=f'Background mean: {formatted_mean[0]}{units[name_series]}')
    ax.hlines(mean[1], year_bkg_end - year_start - 1/2, year_trans_end - year_start - 1/2, linewidth=2, color=colorcycle[1],
              label=f'Transient mean: {formatted_mean[1]}{units[name_series]}')

    plt.tight_layout()
    locs, labels = plt.xticks()  # Get the current locations and labels.
    dt = int(np.floor(len(locs)/10))
    locs = locs[::dt]
    labels = labels[::dt]
    plt.xticks(locs, labels, rotation=0)
    plt.ylabel(name_series+' ['+units[name_series]+']')
    ax.ticklabel_format(axis='y', style='sci', useMathText=True, scilimits=(-3,3))

    # Show the graph
    plt.legend()
    plt.show()
    plt.close()

    return fig

def plot_box_yearly_stat_from_inputs(name_series, time_file, file_to_plot, year_bkg_end, year_trans_end):
    """ Box plot of yearly statistics of the timeseries, together with mean for the background and transient periods
    
    Parameters
    ----------
    name_series : str
        Name of the quantity to plot, has to be one of 'GST', 'Air temperature', 'Precipitation', 'SWE', 'Water production'
    time_file : netCDF4._netCDF4.Variable
        File where the time index of each datapoint is stored        
    file_to_plot : list
        Mean time series (temp_ground_mean, mean_air_temp, mean_prec, swe_mean, tot_water_prod)
    year_bkg_end : int
        Background period is BEFORE the start of the year corresponding to the variable, i.e. all time stamps before Jan 1st year_bkg_end
    year_trans_end : int
        Same for transient period


    Returns
    -------
    fig : Figure
        Box plot of yearly statistics of the timeseries, together with mean for the background and transient periods
    """

    df_data = data_box_yearly_stat(name_series, time_file, file_to_plot, year_bkg_end, year_trans_end)
    fig = plot_box_yearly_stat(df_data, year_bkg_end, year_trans_end)

    return fig

def atmospheric_data_to_panda(list_time_file, list_time_series, label_plot):
    """ Function returns panda data frame for 'air' timeseries, concatenated all reanalyses
        and averaged over all altitudes
    
    Parameters
    ----------
    list_time_file : list of netCDF4._netCDF4.Variable
        List of files where the time index of each datapoint is stored (air time), one per reanalysis
    list_time_series : list of netCDF4._netCDF4.Variable
        List of time series with air time shape (could be air temperature, precipitation, SW, etc.)
    label_plot : str
        label associated to the plot, if 'Precipitation' or 'Water production', rescales data to mm/day from mm/sec
    
    Returns
    -------
    panda_test : pandas.core.frame.DataFrame
        Panda dataframe of the timeseries value per year
        columns are year, label_plot
    """

    panda_list = [[] for _ in list_time_series]

    for i,l in enumerate(list_time_file):
        # create a panda dataframe with month, day, hour for each timestamp
        panda_list[i] = pd.DataFrame(num2date(l[:], l.units), columns=['date'])
        panda_list[i]['year'] = [j.year for j in panda_list[i]['date']]
        # Note that this is averageing the timeseries over all altitudes
        panda_list[i][label_plot] = np.mean(list_time_series[i], axis=1)
        panda_list[i] = panda_list[i].drop(columns=['date'])

    panda_test = pd.concat(panda_list)

    if label_plot in ['Precipitation', 'Water production']:
        panda_test[label_plot] = panda_test[label_plot]*86400

    return panda_test

def sim_data_to_panda(time_file, time_series, list_valid_sim, label_plot):
    """ Function returns panda data frame for 'air' timeseries, concatenated all reanalyses
        and averaged over all altitudes
    
    Parameters
    ----------
    time_file : netCDF4._netCDF4.Variable
        File where the time index of each datapoint is stored (ground time)
    time_series : netCDF4._netCDF4.Variable
        List of time series with ground time shape (could be ground temperature, SWE, etc.)
    list_valid_sim : list
        List of the indices of all valid simulations
    label_plot : str
        label associated to the plot
    
    Returns
    -------
    panda_test : pandas.core.frame.DataFrame
        Panda dataframe of the timeseries value per year
        columns are year, label_plot
    """

    long_timeseries = []
    for sim in list_valid_sim:
        long_timeseries.append(time_series[sim,:,0] if len(time_series.shape) == 3 else time_series[sim,:])

    long_timeseries = np.array(long_timeseries).flatten()
    long_years = np.array([int(i.year) for i in num2date(time_file[:], time_file.units)]*len(list_valid_sim))

    panda_test = pd.DataFrame(long_years.transpose(), columns=['year'])
    panda_test[label_plot] = long_timeseries.transpose()

    return panda_test

def panda_data_to_yearly_stats(panda_test, year_trans_end):
    """ Function returns panda data frame for 'air' timeseries, concatenated all reanalyses
        and averaged over all altitudes
    
    Parameters
    ----------
    panda_test : pandas.core.frame.DataFrame
        Panda dataframe of the timeseries value per year, month, and day
    year_trans_end : int
        Transient period is BEFORE the start of the year corresponding to the variable
    
    Returns
    -------
    yearly_quantiles : pandas.core.frame.DataFrame
        Panda dataframe of the timeseries [0.023, 0.16, 0.5, 0.84, 0.977] quantiles per year.
        The lenght of the table is #quantiles x #years
        Multi-index : (quantile, year) and column: label_plot from function atmospheric_data_to_panda()
    yearly_mean : pandas.core.series.Series
        Panda dataframe of the mean of the timeseries for each year
        index : year and column: label_plot from function atmospheric_data_to_panda()
    """

    list_quantiles = [0.023, 0.16, 0.5, 0.84, 0.977]

    panda_test = panda_test[panda_test['year'] < year_trans_end]

    yearly_quantiles = panda_test.groupby(['year']).quantile(list_quantiles)
    yearly_quantiles.index.names = ['year','quantile']
    yearly_quantiles = yearly_quantiles.swaplevel()
    yearly_mean = panda_test.groupby(['year']).mean()

    return yearly_quantiles, yearly_mean

def plot_yearly_quantiles(yearly_quantiles, yearly_mean, year_bkg_end, year_trans_end):
    """ Function plots yearly statistics for 'air' timeseries, averaged over all reanalyses and altitudes
    
    Parameters
    ----------
    yearly_quantiles : pandas.core.frame.DataFrame
        Panda dataframe of the timeseries [0.023, 0.16, 0.5, 0.84, 0.977] quantiles per year.
        The lenght of the table is #quantiles x #years
        Multi-index : (quantile, year) and column: label_plot from function atmospheric_data_to_panda()
    yearly_mean : pandas.core.series.Series
        Panda dataframe of the mean of the timeseries for each year
        index : year and column: label_plot from function atmospheric_data_to_panda()
    year_bkg_end : int, optional
        Background period is BEFORE the start of the year corresponding to the variable, i.e. all time stamps before Jan 1st year_bkg_end
    year_trans_end : int, optional
        Same for transient period
    
    Returns
    -------
    fig : Figure
        Plot of yearly statistics for any timeseries. Mean and several quantiles for each year.
    """

    list_quantiles = sorted(np.unique([q for q,_ in yearly_quantiles.index]))
    list_years = sorted(np.unique([y for _,y in yearly_quantiles.index]))
    
    label_plot = yearly_quantiles.columns[0]

    mean_bkg = np.mean(yearly_mean.loc[list_years[0]:year_bkg_end-1])
    mean_trans = np.mean(yearly_mean.loc[year_bkg_end:year_trans_end-1])
    # mean_list = [mean_bkg, mean_trans]

    # exponent = [int(np.floor(np.log10(np.abs(i)))) for i in mean_list]
    # formatted_mean = [f"{m:.2e}" for i, m in enumerate(mean_list) if ((exponent[i] < -1) | (exponent[i]>2)) else float(f"{m:.2f}")]
    formatted_mean = [f"{i:.2f}" for i in [mean_bkg, mean_trans]]

    dict_points = {0: {'alpha': 0.2, 'width': 0.5},
                1: {'alpha': 0.4, 'width': 1.0},
                2: {'alpha': 1.0, 'width': 2.0},
                3: {'alpha': 0.4, 'width': 1.0},
                4: {'alpha': 0.2, 'width': 0.5}}

    fig = plt.subplots()

    plt.scatter(list_years, yearly_mean, color=colorcycle[0], linestyle='None', label='Yearly mean')
    # plt.plot(list_years, yearly_mean, color=colorcycle[0], label='Mean')
    # plt.plot(list_years, yearly_quantiles.iloc[dict_indices_quantiles[list_quantiles[2]]][label_plot], color=colorcycle[0])
    for i in [0,1,3,4]:
        plt.scatter(list_years, yearly_quantiles.loc[[list_quantiles[i]]][label_plot], color=colorcycle[0], alpha=dict_points[i]['alpha'], linewidth=dict_points[i]['width'])
    plt.fill_between(list_years, yearly_quantiles.loc[[list_quantiles[1]]][label_plot], yearly_quantiles.loc[[list_quantiles[3]]][label_plot],
                        alpha = 0.4, color=colorcycle[0], linewidth=1,
                        # label='Quantiles 16-84'
                        )
    plt.fill_between(list_years, yearly_quantiles.loc[[list_quantiles[0]]][label_plot], yearly_quantiles.loc[[list_quantiles[4]]][label_plot],
                        alpha = 0.2, color=colorcycle[0], linewidth=0.5,
                        # label='Quantiles 2.3-97.7'
                        )
    
    plt.hlines(mean_bkg, list_years[0], year_bkg_end, color=colorcycle[1],
               label=f'Background mean: {formatted_mean[0]}{units[label_plot]}')
    plt.hlines(mean_trans,  year_bkg_end, list_years[-1], color=colorcycle[2],
               label=f'Transient mean: {formatted_mean[1]}{units[label_plot]}')

    ylim = plt.gca().get_ylim()

    plt.vlines(year_bkg_end, ylim[0], ylim[1], color='grey', linestyle='dashed')

    plt.gca().set_ylim(ylim)

    if label_plot in ['GST', 'Air temperature']:
        plt.axhline(y=0, color='grey', linestyle='dashed')
    
    plt.ylabel(label_plot+' ['+units[label_plot]+']')

    locs = np.arange(list_years[0], list_years[-1]+1, np.floor((list_years[-1]+1-list_years[0])/8), dtype=int)
    plt.xticks(locs, locs)

    # Show the graph
    plt.legend(loc='upper right')
    plt.show()
    plt.close()

    return fig 

def plot_yearly_quantiles_atmospheric_from_inputs(list_time_file, list_time_series, label_plot, year_bkg_end, year_trans_end):
    """ Function returns panda data frame for atmospheric timeseries, concatenated over all reanalyses
        and averaged over all altitudes
        from intended atmospheric input
    
    Parameters
    ----------
    list_time_file : list of netCDF4._netCDF4.Variable
        List of files where the time index of each datapoint is stored (air time), one per reanalysis
    list_time_series : list of netCDF4._netCDF4.Variable
        List of time series with air time shape (could be air temperature, precipitation, SW, etc.)
    label_plot : str
        label associated to the plot, if 'Precipitation' or 'Water production', rescales data to mm/day from mm/sec
    year_bkg_end : int, optional
        Background period is BEFORE the start of the year corresponding to the variable, i.e. all time stamps before Jan 1st year_bkg_end
    year_trans_end : int, optional
        Same for transient period
    
    Returns
    -------
    fig : Figure
        Plot of yearly statistics for atmospheric timeseries. Mean and several quantiles for each year.
    """

    panda_test = atmospheric_data_to_panda(list_time_file, list_time_series, label_plot)
    yearly_quantiles, yearly_mean = panda_data_to_yearly_stats(panda_test, year_trans_end)
    fig = plot_yearly_quantiles(yearly_quantiles, yearly_mean, year_bkg_end, year_trans_end)

    return fig

def plot_yearly_quantiles_sim_from_inputs(time_file, time_series, list_valid_sim, label_plot, year_bkg_end, year_trans_end):
    """ Function returns panda data frame for atmospheric timeseries, concatenated over all reanalyses
        and averaged over all altitudes
        from intended simulated input
    
    Parameters
    ----------
    time_file : netCDF4._netCDF4.Variable
        File where the time index of each datapoint is stored (ground time)
    time_series : netCDF4._netCDF4.Variable
        List of time series with ground time shape (could be ground temperature, SWE, etc.)
    list_valid_sim : list
        List of the indices of all valid simulations
    label_plot : str
        label associated to the plot
    year_bkg_end : int, optional
        Background period is BEFORE the start of the year corresponding to the variable, i.e. all time stamps before Jan 1st year_bkg_end
    year_trans_end : int, optional
        Same for transient period
    
    Returns
    -------
    fig : Figure
        Plot of yearly statistics for simulated timeseries. Mean and several quantiles for each year.
    """

    panda_test = sim_data_to_panda(time_file, time_series, list_valid_sim, label_plot)
    yearly_quantiles, yearly_mean = panda_data_to_yearly_stats(panda_test, year_trans_end)
    fig = plot_yearly_quantiles(yearly_quantiles, yearly_mean, year_bkg_end, year_trans_end)

    return fig

# def plot_yearly_quantiles_all_sims(time_file, time_series, list_valid_sim, label_plot, year_bkg_end, year_trans_end):
#     """ Function plots yearly statistics for 'ground' timeseries over all simulations
    
#     Parameters
#     ----------
#     time_file : netCDF4._netCDF4.Variable
#         File where the time index of each datapoint is stored (ground time)
#     time_series : netCDF4._netCDF4.Variable
#         List of time series with ground time shape (could be ground temperature, SWE, etc.)
#     list_valid_sim : list
#         List of the indices of all valid simulations
#     label_plot : str
#         label associated to the plot.
#     year_bkg_end : int, optional
#         Background period is BEFORE the start of the year corresponding to the variable, i.e. all time stamps before Jan 1st year_bkg_end
#     year_trans_end : int, optional
#         Same for transient period
    
#     Returns
#     -------
#     Plot of yearly statistics for 'ground' timeseries. Mean and several quantiles for each year.
#     """
    
#     dict_points = {0: {'alpha': 0.2, 'width': 0.5},
#                 1: {'alpha': 0.4, 'width': 1.0},
#                 2: {'alpha': 1.0, 'width': 2.0},
#                 3: {'alpha': 0.4, 'width': 1.0},
#                 4: {'alpha': 0.2, 'width': 0.5}}

#     list_quantiles = [0.023, 0.16, 0.5, 0.84, 0.977]
    
#     long_timeseries = []
#     for sim in list_valid_sim:
#         long_timeseries.append(time_series[sim,:,0] if len(time_series.shape) == 3 else time_series[sim,:])

#     long_timeseries = np.array(long_timeseries).flatten()
#     long_years = np.array([int(i.year) for i in num2date(time_file[:], time_file.units)]*len(list_valid_sim))

#     panda_test = pd.DataFrame(long_years.transpose(), columns=['year'])
#     panda_test['timeseries'] = long_timeseries.transpose()

#     list_drop = [i for i in np.unique(panda_test['year']) if i >= year_trans_end]

#     quantiles = panda_test.groupby(['year']).quantile(list_quantiles).drop(index=list_drop)
#     quantiles.index.names = ['year','quantile']
#     quantiles = quantiles.swaplevel()
#     dict_indices_quantiles = quantiles.groupby(['quantile']).indices
#     mean_end = panda_test.groupby(['year']).mean().drop(index=list_drop)
#     xdata = np.array(mean_end.index)

#     mean_bkg = np.mean(mean_end.loc[xdata[0]:year_bkg_end-1])
#     mean_trans = np.mean(mean_end.loc[year_bkg_end:year_trans_end-1])
#     formatted_mean = [f"{i:.2f}" for i in [mean_bkg, mean_trans]]
    
#     plt.scatter(xdata, mean_end, color=colorcycle[0], linestyle='None', label='Yearly mean')
#     # plt.plot(xdata, mean_end, color=colorcycle[0], label='Mean')
#     # plt.plot(xdata, quantiles.iloc[dict_indices_quantiles[list_quantiles[2]]]['timeseries'], color=colorcycle[0])
#     for i in [0,1,3,4]:
#         plt.scatter(xdata, quantiles.iloc[dict_indices_quantiles[list_quantiles[i]]]['timeseries'], color=colorcycle[0], alpha=dict_points[i]['alpha'], linewidth=dict_points[i]['width'])
#     plt.fill_between(xdata, quantiles.iloc[dict_indices_quantiles[list_quantiles[1]]]['timeseries'], quantiles.iloc[dict_indices_quantiles[list_quantiles[3]]]['timeseries'],
#                         alpha = 0.4, color=colorcycle[0], linewidth=1,
#                         # label='Quantiles 16-84'
#                         )
#     plt.fill_between(xdata, quantiles.iloc[dict_indices_quantiles[list_quantiles[0]]]['timeseries'], quantiles.iloc[dict_indices_quantiles[list_quantiles[4]]]['timeseries'],
#                         alpha = 0.2, color=colorcycle[0], linewidth=0.5,
#                         # label='Quantiles 2.3-97.7'
#                         )
#     if label_plot in ['GST', 'Air temperature']:
#         plt.axhline(y=0, color='grey', linestyle='dashed')

#     plt.hlines(mean_bkg, xdata[0], year_bkg_end, color=colorcycle[1],
#                label=f'Background mean: {formatted_mean[0]}{units[label_plot]}')
#     plt.hlines(mean_trans,  year_bkg_end, xdata[-1], color=colorcycle[2],
#                label=f'Transient mean: {formatted_mean[1]}{units[label_plot]}')

#     ylim = plt.gca().get_ylim()
#     plt.vlines(year_bkg_end, ylim[0], ylim[1], color='grey', linestyle='dashed')
#     plt.gca().set_ylim(ylim)

#     plt.ylabel(label_plot+' ['+units[label_plot]+']')

#     plt.tight_layout()  # otherwise the right y-label is slightly clipped

#     # Show the graph
#     plt.legend(loc='upper right')
#     plt.show()
#     plt.close()

def plot_yearly_quantiles_side_by_side(list_yearly_quantiles, list_yearly_mean, list_site, year_bkg_end, year_trans_end):
    """ Function plots yearly statistics for 'air' timeseries, averaged over all reanalyses and altitudes
    
    Parameters
    ----------
    list_yearly_quantiles : list of pandas.core.frame.DataFrame
        List of panda dataframes of the timeseries [0.023, 0.16, 0.5, 0.84, 0.977] quantiles per year.
        The lenght of the table is #quantiles x #years
        Multi-index : (quantile, year) and column: label_plot from function atmospheric_data_to_panda()
    list_yearly_mean : list of pandas.core.series.Series
        List of panda dataframes of the mean of the timeseries for each year
        index : year and column: label_plot from function atmospheric_data_to_panda()
    list_site : list
        List of labels for the site of each entry
    year_bkg_end : int, optional
        Background period is BEFORE the start of the year corresponding to the variable, i.e. all time stamps before Jan 1st year_bkg_end
    year_trans_end : int, optional
        Same for transient period
    
    Returns
    -------
    fig : Figure
        Plot of 2 side by side subplots of yearly statistics for the same timeseries at two different sites.
        Mean and several quantiles for each year.
    """

    list_quantiles = sorted(np.unique([q for q,_ in list_yearly_quantiles[0].index]))
    list_years = sorted(np.unique([y for _,y in list_yearly_quantiles[0].index]))
    
    label_plot = [i.columns[0] for i in list_yearly_quantiles]

    mean_bkg = [np.mean(i.loc[list_years[0]:year_bkg_end-1]) for i in list_yearly_mean]
    mean_trans = [np.mean(i.loc[year_bkg_end:year_trans_end-1]) for i in list_yearly_mean]
    # mean_list = [mean_bkg, mean_trans]

    # exponent = [int(np.floor(np.log10(np.abs(i)))) for i in mean_list]
    # formatted_mean = [f"{m:.2e}" for i, m in enumerate(mean_list) if ((exponent[i] < -1) | (exponent[i]>2)) else float(f"{m:.2f}")]
    formatted_mean = [[f"{i:.2f}" for i in [mean_bkg[j], mean_trans[j]]] for j in range(2)]

    dict_points = {0: {'alpha': 0.2, 'width': 0.5},
                1: {'alpha': 0.4, 'width': 1.0},
                2: {'alpha': 1.0, 'width': 2.0},
                3: {'alpha': 0.4, 'width': 1.0},
                4: {'alpha': 0.2, 'width': 0.5}}

    fig, a = plt.subplots(1, 2, figsize=(8, 4), sharey='row')
    for idx,ax in enumerate(a):
        ax.scatter(list_years, list_yearly_mean[idx], color=colorcycle[idx], linestyle='None', label='Yearly mean')
        # ax.plot(xdata, mean_end, color=colorcycle[idx], label='Mean')
        # plt.plot(xdata, quantiles.iloc[dict_indices_quantiles[list_quantiles[2]]]['timeseries'], color=colorcycle[0])
        for i in [0,1,3,4]:
            ax.scatter(list_years, list_yearly_quantiles[idx].loc[[list_quantiles[i]]][label_plot[idx]], color=colorcycle[idx], alpha=dict_points[i]['alpha'], linewidth=dict_points[i]['width'])
        ax.fill_between(list_years, list_yearly_quantiles[idx].loc[[list_quantiles[1]]][label_plot[idx]], list_yearly_quantiles[idx].loc[[list_quantiles[3]]][label_plot[idx]],
                            alpha = 0.4, color=colorcycle[idx], linewidth=1,
                            # label='Quantiles 16-84'
                            )
        ax.fill_between(list_years, list_yearly_quantiles[idx].loc[[list_quantiles[0]]][label_plot[idx]], list_yearly_quantiles[idx].loc[[list_quantiles[4]]][label_plot[idx]],
                            alpha = 0.2, color=colorcycle[idx], linewidth=0.5,
                            # label='Quantiles 2.3-97.7'
                            )

        ax.hlines(mean_bkg[idx], list_years[0], year_bkg_end, color=colorcycle[2],
               label=f'Background mean: {formatted_mean[idx][0]}{units[label_plot[idx]]}')
        ax.hlines(mean_trans[idx],  year_bkg_end, list_years[-1], color=colorcycle[3],
               label=f'Transient mean: {formatted_mean[idx][1]}{units[label_plot[idx]]}')
            
        if label_plot[idx] in ['GST', 'Air temperature']:
            ax.axhline(y=0, color='grey', linestyle='dashed')

        ax.title.set_text(list_site[idx])
        
        if idx==0:
            ax.set_ylabel(label_plot[idx]+' ['+units[label_plot[idx]]+']')

        ax.legend(loc='upper right' if idx==0 else 'lower right')

    ylim = plt.gca().get_ylim()

    for ax in a:
        ax.vlines(year_bkg_end, ylim[0], ylim[1], color='grey', linestyle='dashed')

    plt.gca().set_ylim(ylim)

    plt.tight_layout()  # otherwise the right y-label is slightly clipped

    # Show the graph
    plt.show()
    plt.close()

    return fig

def plot_yearly_quantiles_side_by_side_atmospheric_from_inputs(list_time_file, list_list_time_series, label_plot, list_site, year_bkg_end, year_trans_end):
    """ Function returns panda data frame for atmospheric timeseries, concatenated over all reanalyses
        and averaged over all altitudes
        from intended atmospheric input
    
    Parameters
    ----------
    list_time_file : list of netCDF4._netCDF4.Variable
        List of files where the time index of each datapoint is stored (air time), one per reanalysis
    list_list_time_series : list of list of netCDF4._netCDF4.Variable
        List (two sites) of list (n reanalyses) of time series with air time shape (could be air temperature, precipitation, SW, etc.)
    label_plot : str
        label associated to the plot, if 'Precipitation' or 'Water production', rescales data to mm/day from mm/sec
    list_site : list
        List of labels for the site of each entry
    year_bkg_end : int, optional
        Background period is BEFORE the start of the year corresponding to the variable, i.e. all time stamps before Jan 1st year_bkg_end
    year_trans_end : int, optional
        Same for transient period
    
    Returns
    -------
    fig : Figure
        Plot of 2 side by side subplots of yearly statistics for the same atmospheric timeseries at two different sites.
        Mean and several quantiles for each year.
    """

    panda_test = [[] for _ in range(2)]
    yearly_quantiles = [[] for _ in range(2)]
    yearly_mean = [[] for _ in range(2)]

    for i in range(2):
        panda_test[i] = atmospheric_data_to_panda(list_time_file, list_list_time_series[i], label_plot[i])
        yearly_quantiles[i], yearly_mean[i] = panda_data_to_yearly_stats(panda_test[i], year_trans_end)
    
    fig = plot_yearly_quantiles_side_by_side(yearly_quantiles, yearly_mean, list_site, year_bkg_end, year_trans_end)

    return fig

def plot_yearly_quantiles_side_by_side_sim_from_inputs(time_file, list_time_series, list_list_valid_sim, label_plot, list_site, year_bkg_end, year_trans_end):
    """ Function returns panda data frame for simulated timeseries, concatenated over all reanalyses
        and averaged over all altitudes
        from intended simulated input
    
    Parameters
    ----------
    time_file : netCDF4._netCDF4.Variable
        File where the time index of each datapoint is stored (ground time)
    list_time_series : netCDF4._netCDF4.Variable
        List of time series with ground time shape (could be ground temperature, SWE, etc.)
    list_list_valid_sim : list of list
        List (2 sites) of list of indices of all valid simulations
    label_plot : str
        label associated to the plot
    list_site : list
        List of labels for the site of each entry
    year_bkg_end : int, optional
        Background period is BEFORE the start of the year corresponding to the variable, i.e. all time stamps before Jan 1st year_bkg_end
    year_trans_end : int, optional
        Same for transient period
    
    Returns
    -------
    Plot of 2 side by side subplots of yearly statistics for the same simulated timeseries at two different sites.
        Mean and several quantiles for each year.
    """

    panda_test = [[] for _ in range(2)]
    yearly_quantiles = [[] for _ in range(2)]
    yearly_mean = [[] for _ in range(2)]

    for i in range(2):
        panda_test[i] = sim_data_to_panda(time_file, list_time_series[i], list_list_valid_sim[i], label_plot)
        yearly_quantiles[i], yearly_mean[i] = panda_data_to_yearly_stats(panda_test[i], year_trans_end)
    
    fig = plot_yearly_quantiles_side_by_side(yearly_quantiles, yearly_mean, list_site, year_bkg_end, year_trans_end)

    return fig

# def plot_yearly_quantiles_all_sims_side_by_side(time_file, time_series, list_valid_sim, label_plot, list_site, year_bkg_end, year_trans_end):
#     """ Function plots yearly statistics for 'ground' timeseries over all simulations for 1 same metric
#         at 2 different sites. 1 plot per site, both plots side by side.
    
#     Parameters
#     ----------
#     time_file : netCDF4._netCDF4.Variable
#         File where the time index of each datapoint is stored (ground time)
#     time_series : netCDF4._netCDF4.Variable
#         List of time series with ground time shape (could be ground temperature, SWE, etc.)
#     list_valid_sim : list
#         List of the indices of all valid simulations
#     label_plot : str
#         label associated to the plot.
#     list_site : list
#         List of labels for the site of each entry
#     year_bkg_end : int, optional
#         Background period is BEFORE the start of the year corresponding to the variable, i.e. all time stamps before Jan 1st year_bkg_end
#     year_trans_end : int, optional
#         Same for transient period
    
#     Returns
#     -------
#     Plots of yearly statistics for 'ground' timeseries over all simulations for 1 same metric
#     at 2 different sites. 1 plot per site, both plots side by side.
#     """
    
#     dict_points = {0: {'alpha': 0.2, 'width': 0.5},
#                 1: {'alpha': 0.4, 'width': 1.0},
#                 2: {'alpha': 1.0, 'width': 2.0},
#                 3: {'alpha': 0.4, 'width': 1.0},
#                 4: {'alpha': 0.2, 'width': 0.5}}

#     list_quantiles = [0.023, 0.16, 0.5, 0.84, 0.977]

#     _, a = plt.subplots(1, 2, figsize=(8, 4), sharey='row')
#     for idx,ax in enumerate(a):
#         long_timeseries = []
#         for sim in list_valid_sim[idx]:
#             long_timeseries.append(time_series[idx][sim,:,0] if len(time_series[idx].shape) == 3 else time_series[idx][sim,:])

#         long_timeseries = np.array(long_timeseries).flatten()
#         long_years = np.array([int(i.year) for i in num2date(time_file[:], time_file.units)]*len(list_valid_sim[idx]))

#         panda_test = pd.DataFrame(long_years.transpose(), columns=['year'])
#         panda_test['timeseries'] = long_timeseries.transpose()

#         list_drop = [i for i in np.unique(panda_test['year']) if i >= year_trans_end]

#         quantiles = panda_test.groupby(['year']).quantile(list_quantiles).drop(index=list_drop)
#         quantiles.index.names = ['year','quantile']
#         quantiles = quantiles.swaplevel()
#         dict_indices_quantiles = quantiles.groupby(['quantile']).indices
#         mean_end = panda_test.groupby(['year']).mean().drop(index=list_drop)
#         xdata = np.array(mean_end.index)

#         mean_bkg = np.mean(mean_end.loc[xdata[0]:year_bkg_end-1])
#         mean_trans = np.mean(mean_end.loc[year_bkg_end:year_trans_end-1])
#         formatted_mean = [f"{i:.2f}" for i in [mean_bkg, mean_trans]]
        
#         ax.scatter(xdata, mean_end, color=colorcycle[idx], linestyle='None', label='Yearly mean')
#         # ax.plot(xdata, mean_end, color=colorcycle[idx], label='Mean')
#         # plt.plot(xdata, quantiles.iloc[dict_indices_quantiles[list_quantiles[2]]]['timeseries'], color=colorcycle[0])
#         for i in [0,1,3,4]:
#             ax.scatter(xdata, quantiles.iloc[dict_indices_quantiles[list_quantiles[i]]]['timeseries'], color=colorcycle[idx], alpha=dict_points[i]['alpha'], linewidth=dict_points[i]['width'])
#         ax.fill_between(xdata, quantiles.iloc[dict_indices_quantiles[list_quantiles[1]]]['timeseries'], quantiles.iloc[dict_indices_quantiles[list_quantiles[3]]]['timeseries'],
#                             alpha = 0.4, color=colorcycle[idx], linewidth=1,
#                             # label='Quantiles 16-84'
#                             )
#         ax.fill_between(xdata, quantiles.iloc[dict_indices_quantiles[list_quantiles[0]]]['timeseries'], quantiles.iloc[dict_indices_quantiles[list_quantiles[4]]]['timeseries'],
#                             alpha = 0.2, color=colorcycle[idx], linewidth=0.5,
#                             # label='Quantiles 2.3-97.7'
#                             )

#         ax.hlines(mean_bkg, xdata[0], year_bkg_end, color=colorcycle[2],
#                label=f'Background mean: {formatted_mean[0]}{units[label_plot]}')
#         ax.hlines(mean_trans,  year_bkg_end, xdata[-1], color=colorcycle[3],
#                label=f'Transient mean: {formatted_mean[1]}{units[label_plot]}')
            
#         if label_plot in ['GST', 'Air temperature']:
#             ax.axhline(y=0, color='grey', linestyle='dashed')

#         ax.title.set_text(list_site[idx])
        
#         if idx==0:
#             ax.set_ylabel(label_plot+' ['+units[label_plot]+']')

#         ax.legend(loc='upper right' if idx==0 else 'lower right')

#     ylim = plt.gca().get_ylim()

#     for ax in a:
#         ax.vlines(year_bkg_end, ylim[0], ylim[1], color='grey', linestyle='dashed')

#     plt.gca().set_ylim(ylim)

#     plt.tight_layout()  # otherwise the right y-label is slightly clipped

#     # Show the graph
#     plt.show()
#     plt.close()

def plot_sanity_two_variables_one_year_quantiles_side_by_side(time_file, time_series_list, list_valid_sim_list, list_label, list_site):
    """ Function returns 2 plots side by side of 2 timeseries each reduced to a 1-year window with mean and 1 and 2-sigma spread.
        Each plot is a plot of two timeseries of the same variable at two different sites
    
    Parameters
    ----------
    time_file : netCDF4._netCDF4.Variable
        File where the time index of each datapoint is stored (time_ground, not time_air)
    time_series_list : list of netCDF4._netCDF4.Variable
        List of time series (could be temperature, precipitation, snow depth, etc.). 1 per plot.
    list_valid_sim_list : list of list
        List of list of the indices of all valid simulations
    list_label : list
        List of the labels associated to each plot
    list_site : list
        List of labels for the site of each entry

    Returns
    -------
    Plot of 2 (or more?) timeseries reduced to a 1-year window with mean and 1 and 2-sigma spread.
    Both series have their own y axis if they have different units.

    """

    _, a = plt.subplots(1, 2, figsize=(10, 5))
    for idx,ax in enumerate(a):

        quantiles, mean_end = stats_all_years_simulations_to_single_year(time_file, time_series_list[idx][0], list_valid_sim_list[0])
        xdata = range(len(mean_end))

        indx=0
        ax.plot(xdata, mean_end, color=colorcycle[indx], linewidth=2, label=list_site[indx])
        ax.fill_between(xdata, quantiles.iloc[1], quantiles.iloc[3], alpha = 0.4, color=colorcycle[indx], linewidth=1)
        ax.fill_between(xdata, quantiles.iloc[0], quantiles.iloc[4], alpha = 0.2, color=colorcycle[indx], linewidth=0.5)
        ax.set_ylabel(list_label[idx]+' ['+units[list_label[idx]]+']')

        quantiles, mean_end = stats_all_years_simulations_to_single_year(time_file, time_series_list[idx][1], list_valid_sim_list[1])
        indx=1

        ax.plot(xdata, mean_end, color=colorcycle[indx], linewidth=2, label=list_site[indx])
        ax.fill_between(xdata, quantiles.iloc[1], quantiles.iloc[3], alpha = 0.4, color=colorcycle[indx], linewidth=1)
        ax.fill_between(xdata, quantiles.iloc[0], quantiles.iloc[4], alpha = 0.2, color=colorcycle[indx], linewidth=0.5)

        if list_label[idx] in ['GST', 'Air temperature']:
            ax.axhline(y=0, color='grey', linestyle='dashed')

        locs = np.linspace(0, len(mean_end), num=13, endpoint=True)
        labels = ['Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec','Jan']
        ax.set_xticks(locs)
        ax.set_xticklabels(labels)

        if idx == 1:
            ax.legend(loc="upper right")


    plt.tight_layout()  # otherwise the right y-label is slightly clipped

    # Show the graph
    plt.show()
    plt.close()



"""This module defines the functions that will open the netCDF (.nc) files"""

#pylint: disable=invalid-name

from netCDF4 import Dataset #pylint: disable=no-name-in-module

def open_air_nc(path_forcing):
    """ Function returns data from the .nc file for a given atmospheric forcing
    
    Parameters
    ----------
    path_forcing : str
        Path to the .nc file where the atmospheric forcing data is stored

    Returns
    -------
    time_air : netCDF4._netCDF4.Variable
        Time file for the atmospheric forcing in the shape (time)
    temp_air : netCDF4._netCDF4.Variable
        Air temperature in the shape (time, station)
    SW_flux : netCDF4._netCDF4.Variable
        Shortwave (SW) flux in the shape (time, station)
    SW_direct_flux : netCDF4._netCDF4.Variable
        Direct shortwave (SW) in the shape (time, station)
    SW_diffuse_flux : netCDF4._netCDF4.Variable
        Diffuse shortwave (SW) in the shape (time, station)
    precipitation : netCDF4._netCDF4.Variable
        Precipitation in the shape (time, station)
    """

    # Open file for air temperature
    ncfile_air = Dataset(path_forcing, mode='r')

    time_air = ncfile_air['time']
    temp_air = ncfile_air['AIRT_pl']
    SW_flux = ncfile_air['SW_sur']
    SW_direct_flux = ncfile_air['SW_topo_direct']
    SW_diffuse_flux = ncfile_air['SW_topo_diffuse']
    precipitation = ncfile_air['PREC_sur']

    return time_air, temp_air, SW_flux, SW_direct_flux, SW_diffuse_flux, precipitation

def open_ground_nc(path_ground):
    """ Function returns data from the .nc file for the ground simulations
    
    Parameters
    ----------
    path_ground : str
        Path to the .nc file where the aggregated ground simulations are stored

    Returns
    -------
    f_ground : netCDF4._netCDF4.Group
        geotop netCDF group with description of dimensions, variables, and data for the ground
    time_ground : netCDF4._netCDF4.Variable
        Time file for the ground simulations in the shape (time)
    temp_ground : netCDF4._netCDF4.Variable
        Ground temperature in the shape (simulation, time, soil_depth)
    """

    # Open file for ground temperature
    ncfile_ground = Dataset(path_ground, mode='r')
    # Select geotop model data
    f_ground = ncfile_ground.groups['geotop']

    time_ground = f_ground['Date']
    temp_ground = f_ground['Tg']

    return f_ground, time_ground, temp_ground

def open_snow_nc(path_snow):
    """ Function returns data from the .nc file for the snow simulations
    
    Parameters
    ----------
    path_snow : str
        Path to the .nc file where the aggregated snow simulations are stored

    Returns
    -------
    f_snow : netCDF4._netCDF4.Group
        geotop netCDF group with description of dimensions, variables, and data for the snow
    snow_height : netCDF4._netCDF4.Variable
        Depth of snow in the shape (simulation, time)
    """

    # Open file for snow depth
    ncfile_snow = Dataset(path_snow, mode='r')
    # Select geotop model data
    f_snow = ncfile_snow.groups['geotop']

    snow_height = f_snow['snow_depth_mm']

    return f_snow, snow_height

def open_swe_nc(path_swe):
    """ Function returns data from the .nc file
    for the snow water equivalent (SWE) simulation results
    
    Parameters
    ----------
    path_swe : str
        Path to the .nc file where the aggregated SWE simulations are stored

    Returns
    -------
    f_swe : netCDF4._netCDF4.Group
        geotop netCDF group with description of dimensions, variables, and data for the SWE
    swe : netCDF4._netCDF4.Variable
        SWE in the shape (simulation, time)
    """

    # Open file for snow depth
    ncfile_swe = Dataset(path_swe, mode='r')
    # Select geotop model data
    f_swe = ncfile_swe.groups['geotop']

    swe = f_swe['snow_water_equivalent']

    return f_swe, swe

def open_thaw_depth_nc(path_thaw_depth):
    """ Function returns data from the .nc file for the depth of thaw simulation results
    
    Parameters
    ----------
    path_thaw_depth : str
        Path to the .nc file where the aggregated thaw depth simulations are stored

    Returns
    -------
    f_thaw_depth : netCDF4._netCDF4.Group
        geotop netCDF group with description of dimensions, variables, and data for the thaw depth
    thaw_depth : netCDF4._netCDF4.Variable
        Depth of thaw in the shape (simulation, time)
    """

    # Open file for snow depth
    ncfile_thaw_depth = Dataset(path_thaw_depth, mode='r')
    # Select geotop model data
    f_thaw_depth = ncfile_thaw_depth.groups['geotop']

    thaw_depth = f_thaw_depth['AL']

    return f_thaw_depth, thaw_depth

def open_SW_direct_nc(path_SW_direct):
    """ Function returns data from the .nc file for the SW_direct simulations
    
    Parameters
    ----------
    path_SW_direct : str
        Path to the .nc file where the aggregated SW_direct simulations are stored

    Returns
    -------
    f_SW_direct : netCDF4._netCDF4.Group
        geotop netCDF group with description of dimensions, variables, and data for the SW_direct
    SW_direct_height : netCDF4._netCDF4.Variable
        Depth of SW_direct in the shape (simulation, time)
    """

    # Open file for SW_direct depth
    ncfile_SW_direct = Dataset(path_SW_direct, mode='r')
    # Select geotop model data
    f_SW_direct = ncfile_SW_direct.groups['geotop']

    SW_direct = f_SW_direct['SW_direct']

    return f_SW_direct, SW_direct

def open_SW_diffuse_nc(path_SW_diffuse):
    """ Function returns data from the .nc file for the SW_diffuse simulations
    
    Parameters
    ----------
    path_SW_diffuse : str
        Path to the .nc file where the aggregated SW_diffuse simulations are stored

    Returns
    -------
    f_SW_diffuse : netCDF4._netCDF4.Group
        geotop netCDF group with description of dimensions, variables, and data for the SW_diffuse
    SW_diffuse_height : netCDF4._netCDF4.Variable
        Depth of SW_diffuse in the shape (simulation, time)
    """

    # Open file for SW_diffuse depth
    ncfile_SW_diffuse = Dataset(path_SW_diffuse, mode='r')
    # Select geotop model data
    f_SW_diffuse = ncfile_SW_diffuse.groups['geotop']

    SW_diffuse = f_SW_diffuse['SW_diffuse']

    return f_SW_diffuse, SW_diffuse

def open_SW_up_nc(path_SW_up):
    """ Function returns data from the .nc file for the SW_up simulations
    
    Parameters
    ----------
    path_SW_up : str
        Path to the .nc file where the aggregated SW_up simulations are stored

    Returns
    -------
    f_SW_up : netCDF4._netCDF4.Group
        geotop netCDF group with description of dimensions, variables, and data for the SW_up
    SW_up_height : netCDF4._netCDF4.Variable
        Depth of SW_up in the shape (simulation, time)
    """

    # Open file for SW_up depth
    ncfile_SW_up = Dataset(path_SW_up, mode='r')
    # Select geotop model data
    f_SW_up = ncfile_SW_up.groups['geotop']

    SW_up = f_SW_up['SW_up']

    return f_SW_up, SW_up

def open_SW_down_nc(path_SW_down):
    """ Function returns data from the .nc file for the SW_down simulations
    
    Parameters
    ----------
    path_SW_down : str
        Path to the .nc file where the aggregated SW_down simulations are stored

    Returns
    -------
    f_SW_down : netCDF4._netCDF4.Group
        geotop netCDF group with description of dimensions, variables, and data for the SW_down
    SW_down_height : netCDF4._netCDF4.Variable
        Depth of SW_down in the shape (simulation, time)
    """

    # Open file for SW_down depth
    ncfile_SW_down = Dataset(path_SW_down, mode='r')
    # Select geotop model data
    f_SW_down = ncfile_SW_down.groups['geotop']

    SW_down = f_SW_down['SW_down']

    return f_SW_down, SW_down

def open_SW_net_nc(path_SW_net):
    """ Function returns data from the .nc file for the SW_net simulations
    
    Parameters
    ----------
    path_SW_net : str
        Path to the .nc file where the aggregated SW_net simulations are stored

    Returns
    -------
    f_SW_net : netCDF4._netCDF4.Group
        geotop netCDF group with description of dimensions, variables, and data for the SW_net
    SW_net_height : netCDF4._netCDF4.Variable
        Depth of SW_net in the shape (simulation, time)
    """

    # Open file for SW_net depth
    ncfile_SW_net = Dataset(path_SW_net, mode='r')
    # Select geotop model data
    f_SW_net = ncfile_SW_net.groups['geotop']

    SW_net = f_SW_net['SW_net']

    return f_SW_net, SW_net

def open_LW_net_nc(path_LW_net):
    """ Function returns data from the .nc file for the LW_net simulations
    
    Parameters
    ----------
    path_LW_net : str
        Path to the .nc file where the aggregated LW_net simulations are stored

    Returns
    -------
    f_LW_net : netCDF4._netCDF4.Group
        geotop netCDF group with description of dimensions, variables, and data for the LW_net
    LW_net_height : netCDF4._netCDF4.Variable
        Depth of LW_net in the shape (simulation, time)
    """

    # Open file for LW_net depth
    ncfile_LW_net = Dataset(path_LW_net, mode='r')
    # Select geotop model data
    f_LW_net = ncfile_LW_net.groups['geotop']

    LW_net = f_LW_net['LW_net']

    return f_LW_net, LW_net



"""This module creates plots of the evolution of metrics between background and transient periods"""

#pylint: disable=line-too-long
#pylint: disable=trailing-whitespace
#pylint: disable=invalid-name

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from scipy.stats import linregress

from SuPerSim.pickling import load_all_pickles
from SuPerSim.topoheatmap import table_background_evolution_mean_GST_aspect_slope
from SuPerSim.constants import colorcycle

def sorted_bkg_GST_data(site, path_pickle):
    """ Function returns background GST and GST evolution for a single site.
    The two lists have the same order and are ordered in background GST increasing order
    
    Parameters
    ----------
    site : str
        Location of the event, e.g. 'Joffre' or 'Fingerpost'
    path_pickle : str
        String path to the location of the folder where the pickles are saved
    
    Returns
    -------
    list_xy : list
        List of lists of background GST and GST evolution for a single site.
        The nth element of each list corresponds to the same simulation
        The background GST is in increasing order
        in the shape [list_x, list_y]
    """
    
    pkl = load_all_pickles(site, path_pickle)
    df_stats = pkl['df_stats']

    df_stats_bis = pd.DataFrame(data=df_stats, columns=['bkg_grd_temp', 'evol_grd_temp'])
    df_stats_bis['bkg_grd_temp'] = pd.Categorical(df_stats_bis['bkg_grd_temp'], np.sort(df_stats['bkg_grd_temp']))
    df_stats_bis = df_stats_bis.sort_values('bkg_grd_temp')

    list_xy = [list(df_stats_bis.loc[:, 'bkg_grd_temp']), list(df_stats_bis.loc[:, 'evol_grd_temp'])]

    return list_xy

def plot_GST_bkg_vs_evol_quantile_bins_fit_single_site(list_xy):
    """ Function return scatter plot of background GST vs GST evolution for a single site.
    The site is binned in 10 bins of equal sizes and each bin is represented by a dot with x and y error bars.
    A linear regression is produced too.
    
    Parameters
    ----------
    list_xy : list
        List of lists of background GST and GST evolution for a single site.
        The nth element of each list corresponds to the same simulation
        The background GST is in increasing order
        in the shape [list_x, list_y]
    
    Returns
    -------
    fig : figure
        Scatter plot of background GST vs GST evolution for the single site.
        The single site is binned in 10 bins of equal sizes and each bin is represented by a dot with x and y error bars.
        A linear regression is produced too.
    """

    quantiles = np.arange(0, 101, 10)

    fig, _ = plt.subplots()

    cmap = plt.cm.seismic #pylint: disable=no-member

    list_x, list_y = list_xy

    list_x_mean = []

    for i in range(len(quantiles)-1):
        low = int(np.ceil(len(list_x)*quantiles[i]/100))
        up = int(np.ceil(len(list_x)*quantiles[i+1]/100))
        list_x_mean.append(np.mean(list_x[low:up]))
        # plt.hlines(np.mean(list_y[low:up]),list_x[low],list_x[up-1], color=colorcycle[i], linewidth=2)

    vmax = np.max(np.abs(list_x_mean))

    for i in range(len(quantiles)-1):
        color = cmap((list_x_mean[i] + vmax)/(2*vmax))
        low = int(np.ceil(len(list_x)*quantiles[i]/100))
        up = int(np.ceil(len(list_x)*quantiles[i+1]/100))
        plt.scatter(list_x[low:up], list_y[low:up], color=color,s=0.8)
        plt.errorbar(np.mean(list_x[low:up]), np.mean(list_y[low:up]), np.std(list_y[low:up]), np.std(list_x[low:up]), color=color)
        plt.scatter(np.mean(list_x[low:up]), np.mean(list_y[low:up]), color=color, s=50)
        # plt.hlines(np.mean(list_y[low:up]),list_x[low],list_x[up-1], color=colorcycle[i], linewidth=2)

    slope, intercept, r, _, _ = linregress(list_x, list_y)
    u = np.arange(np.min(list_x)-0.1, np.max(list_x)+0.1, 0.01)
    print('R-square:', f"{r**2:.4f}", ', regression slope:', f"{slope:.4f}" , ', regression intercept:', f"{intercept:.4f}")
    plt.plot(u, slope*u+intercept, c='grey', label=f'slope: {round(slope,3)}')

    plt.xlabel('Mean background GST [°C]')
    plt.ylabel('Mean GST evolution [°C]')

    # Show the graph
    # plt.legend()
    plt.show()
    plt.close()

    return fig

def plot_GST_bkg_vs_evol_quantile_bins_fit_single_site_from_inputs(site, path_pickle):
    """ Function return scatter plot of background GST vs GST evolution for a single site.
    The site is binned in 10 bins of equal sizes and each bin is represented by a dot with x and y error bars.
    A linear regression is produced too.
    
    Parameters
    ----------
    site : str
        Location of the event, e.g. 'Joffre' or 'Fingerpost'
    path_pickle : str
        String path to the location of the folder where the pickles are saved
    
    Returns
    -------
    fig : figure
        Scatter plot of background GST vs GST evolution for the single site.
        The single site is binned in 10 bins of equal sizes and each bin is represented by a dot with x and y error bars.
        A linear regression is produced too.
    """

    list_xy = sorted_bkg_GST_data(site, path_pickle)
    fig =plot_GST_bkg_vs_evol_quantile_bins_fit_single_site(list_xy)

    return fig

def sorted_bkg_GST_data_two_sites(list_site, list_path_pickle):
    """ Function returns background GST and GST evolution for two sites.
        For each site, the two lists have the same order and are ordered in background GST increasing order
    
    Parameters
    ----------
    list_site : list
        List of labels for the site of each entry
    list_path_pickle : list of str
        List of string path to the location of the folder where the pickles are saved
    
    Returns
    -------
    list_xy : list
        List of lists of background GST and GST evolution for a single site.
        The nth element of each list corresponds to the same simulation
        in the shape [[list_x_site0, list_y_site0], [list_x_site1, list_y_site1]]
    """

    list_xy = [[] for _ in list_site]

    num = len(list_site)

    for i in range(num):
        list_xy[i] = sorted_bkg_GST_data(list_site[i], list_path_pickle[i])

    return list_xy

def plot_GST_bkg_vs_evol_quantile_bins_fit_two_sites(list_xy, list_label_site):
    """ Function return scatter plot of background GST vs GST evolution for 2 sites.
        Both sites are binned in 10 bins of equal sizes and each bin is represented by a dot with x and y error bars.
        A linear regression is produced for each site.
    
    Parameters
    ----------
    list_xy : list
        List of lists of background GST and GST evolution for a single site.
        The nth element of each list corresponds to the same simulation
        in the shape [[list_x_site0, list_y_site0], [list_x_site1, list_y_site1]]
    list_label_site : list of str
        List of label for each site
    
    Returns
    -------
    fig : figure
        List of linear regression r-value (1 for each site). Need to square it to get R^2.
        Scatter plot of background GST vs GST evolution for 2 sites.
        Both sites are binned in 10 bins of equal sizes and each bin is represented by a dot with x and y error bars.
        A linear regression is produced for each site.
    """

    num = len(list_xy)

    list_x = [[] for _ in list_xy]
    list_y = [[] for _ in list_xy]

    for i in range(num):
        list_x[i], list_y[i] = list_xy[i]

    quantiles = np.arange(0, 101, 10)

    fig, _ = plt.subplots()

    cmap = plt.cm.seismic #pylint: disable=no-member
    # colors = cmap(np.linspace(0, 1, len(quantiles)+(1 if len(quantiles)%2 else 0)))

    vmax = [[] for _ in list_xy]
    list_x_mean = [[] for _ in list_xy]

    for j in range(num):

        for i in range(len(quantiles)-1):
            low = int(np.ceil(len(list_x[j])*quantiles[i]/100))
            up = int(np.ceil(len(list_x[j])*quantiles[i+1]/100))
            list_x_mean[j].append(np.mean(list_x[j][low:up]))

        vmax[j] = np.max(np.abs(list_x_mean[j]))

    vmax = np.max(vmax)

    points = []
    slope = []
    intercept = []
    r = []

    for j in range(num):

        for i in range(len(quantiles)-1):
            color = cmap((list_x_mean[j][i] + vmax)/(2*vmax))
            low = int(np.ceil(len(list_x[j])*quantiles[i]/100))
            up = int(np.ceil(len(list_x[j])*quantiles[i+1]/100))
            plt.scatter(list_x[j][low:up], list_y[j][low:up], color=color,s=0.8)
            plt.errorbar(np.mean(list_x[j][low:up]), np.mean(list_y[j][low:up]), np.std(list_y[j][low:up]), np.std(list_x[j][low:up]), color=color)
            plt.scatter(np.mean(list_x[j][low:up]), np.mean(list_y[j][low:up]), color=color, s=50)
            # plt.hlines(np.mean(list_y[low:up]),list_x[low],list_x[up-1], color=colorcycle[i], linewidth=2)

        slope_i, intercept_i, r_i, _, _ = linregress(list_x[j], list_y[j])
        slope.append(slope_i)
        intercept.append(intercept_i)
        r.append(r_i)
        u = np.arange(np.min(list_x[j])-0.05, np.max(list_x[j])+0.05, 0.01)
        print('R-square:', r_i**2, ', regression slope:', slope_i , ', regression intercept:', intercept_i)
        line_j, = plt.plot(u, slope_i*u+intercept_i, c=colorcycle[j], label=list_label_site[j])
        if j==0:
            first_legend = plt.legend(handles=[line_j], loc='upper left')

        xpts = np.max(list_x[j])+0.05 if j==0 else np.min(list_x[j])-0.05
        ypts = slope_i*xpts+intercept_i

        points.append([xpts, ypts])

    xlim = plt.gca().get_xlim()
    ylim = plt.gca().get_ylim()

    u = np.arange(xlim[0], xlim[1], 0.01)
    slope_divide = (points[0][1]-points[1][1])/(points[0][0]-points[1][0])
    intercept_divide = points[1][1] - slope_divide*points[1][0]
    line = slope_divide*u+intercept_divide
    plt.plot(u, line, c='grey', linestyle='dashed')

    plt.fill_between(u, ylim[0]*len(u), line, alpha = 0.1, color=colorcycle[0], linewidth=1)
    plt.fill_between(u, line, ylim[1]*len(u), alpha = 0.1, color=colorcycle[1], linewidth=1)

    plt.plot()

    plt.gca().set_xlim(xlim)
    plt.gca().set_ylim(ylim)

    plt.xlabel('Mean background GST [°C]')
    plt.ylabel('Mean GST evolution [°C]')

    # Show the graph
    # first_legend = plt.legend(handles=[line[0]], loc='upper left')
    plt.gca().add_artist(first_legend)
    plt.legend(handles=[line_j], loc='lower right')
    plt.show()
    plt.close()

    return fig

def plot_GST_bkg_vs_evol_quantile_bins_fit_two_sites_from_input(list_site, list_path_pickle, list_label_site):
    """ Function return scatter plot of background GST vs GST evolution for 2 sites.
    Both sites are binned in 10 bins of equal sizes and each bin is represented by a dot with x and y error bars.
    A linear regression is produced for each site.
    
    Parameters
    ----------
    list_site : list
        List of labels for the site of each entry
    list_path_pickle : list of str
        List of string path to the location of the folder where the pickles are saved
    list_label_site : list of str
        List of label for each site
    
    Returns
    -------
    fig : figure
        List of linear regression r-value (1 for each site). Need to square it to get R^2.
        Scatter plot of background GST vs GST evolution for 2 sites.
        Both sites are binned in 10 bins of equal sizes and each bin is represented by a dot with x and y error bars.
        A linear regression is produced for each site.
    """

    list_xy = sorted_bkg_GST_data_two_sites(list_site, list_path_pickle)
    fig = plot_GST_bkg_vs_evol_quantile_bins_fit_two_sites(list_xy, list_label_site)

    return fig

def data_bkg_GST_evol_altitudes(site, path_pickle):
    """ Function returns background GST and GST evolution for a single site for each altitude
    
    Parameters
    ----------
    site : str
        Location of the event, e.g. 'Joffre' or 'Fingerpost'
    path_pickle : str
        String path to the location of the folder where the pickles are saved


    Returns
    -------
    dic_data : dict
        Dictionary with background GST and GST evolution for each altitude with keys ['bkg', 'evol']
        in the form e.g.
        {'bkg': {2900: [...], 3100: [...], ...}, 'evol': {2900: [...], 3100: [...], ...}}
    """

    table_all = table_background_evolution_mean_GST_aspect_slope(site, path_pickle)
    dic_data = {period: {k: [i for j in v.columns for i in v[j]] for k, v in table_all[period].items()}
                for period in ['bkg', 'evol']}
    
    return dic_data
    
def plot_mean_bkg_GST_vs_evolution(dic_data):
    """ Function returns a scatter plot of mean background GST (ground-surface temperature)
        vs evolution of mean GST between the background and transient period.
        Note that each point is computed from an average over all reanalyses to avoid bias.
    
    Parameters
    ----------
    site : str
        Location of the event, e.g. 'Joffre' or 'Fingerpost'
    path_pickle : str
        String path to the location of the folder where the pickles are saved


    Returns
    -------
    fig : figure
        Scatter plot
    """

    xx = [i for i in dic_data['bkg'].values()]
    yy = [i for i in dic_data['evol'].values()]

    alt_list = list(dic_data['bkg'].keys())

    fig, _ = plt.subplots()

    for i,x in enumerate(xx):
        slope, intercept, r, _, _ = linregress(x,yy[i])
        print('altitude:', alt_list[i], ', R-square:', f"{r**2:.4f}", ', regression slope:', f"{slope:.4f}", ', regression intercept:', f"{intercept:.4f}")
        u = np.arange(np.min(x)-0.1, np.max(x)+0.1, 0.01)
        plt.scatter(x,yy[i], c=colorcycle[i], label=f'{alt_list[i]} m')
        plt.plot(u, slope*u+intercept, c=colorcycle[i], label=f'slope: {round(slope,3)}')

    plt.legend(loc='lower left')
    plt.xlabel('Mean background GST [°C]')
    plt.ylabel('Mean GST evolution [°C]')

    # displaying the scatter plot
    plt.show()
    plt.close()

    return fig

def plot_mean_bkg_GST_vs_evolution_from_inputs(site, path_pickle):
    """ Function returns a scatter plot of mean background GST (ground-surface temperature)
        vs evolution of mean GST between the background and transient period.
        Note that each point is computed from an average over all reanalyses to avoid bias.
    
    Parameters
    ----------
    site : str
        Location of the event, e.g. 'Joffre' or 'Fingerpost'
    path_pickle : str
        String path to the location of the folder where the pickles are saved


    Returns
    -------
    fig : figure
        Scatter plot
    """

    dic_data = data_bkg_GST_evol_altitudes(site, path_pickle)
    fig = plot_mean_bkg_GST_vs_evolution(dic_data)

    return fig

def data_evolution_snow_cover_melt_out(site, path_pickle, variable=None, value=None):
    """ Function returns lists of evolution of snow cover (in days) and melt out date
        between background and transient periods
        For all simulations or for a given subset, for instance the ones with 'slope'=55
        Note that only the simulations with snow are accounted for,
        otherwise, we would get a huge spike at 0 for all simulations that had no snow and kept it this way.

    Parameters
    ----------
    site : str
        Location of the event, e.g. 'Joffre' or 'Fingerpost'
    path_pickle : str
        String path to the location of the folder where the pickles are saved
    variable : str
        A parameter in df_stats such as slope, aspect, altitude, etc.
    value : float
        A value for the parameter, e.g. 50 for a slope

    Returns
    -------
    evol_melt_out : list
        List of the evolution of melt out date
    evol_snow_cover : list
        List of the evolution of snow cover
    """
    
    pkl = load_all_pickles(site, path_pickle)
    df_stats = pkl['df_stats']

    # creates a subset of df_stats given the value of the variable entered as input. e.g. 'slope'=50
    if variable is None:
        data = df_stats
    else:
        data = df_stats[df_stats[variable]==value]

    # creates a list of the time evolution of both parameters
    # makes sure to only keep the simulations that have shown at least 1 day of snow over the whole study period
    evol_melt_out = [data.iloc[k].melt_out_trans - data.iloc[k].melt_out_bkg for k in range(len(data)) if (data.iloc[k].frac_snow_bkg != 0) | (data.iloc[k].frac_snow_trans != 0)]
    evol_snow_cover = [(data.iloc[k].frac_snow_trans - data.iloc[k].frac_snow_bkg)*365.25 for k in range(len(data)) if (data.iloc[k].frac_snow_bkg != 0) | (data.iloc[k].frac_snow_trans != 0)]

    return evol_melt_out, evol_snow_cover

def plot_evolution_snow_cover_melt_out(evol_melt_out, evol_snow_cover):
    """ Function returns a histogram of the evolution of snow cover (in days) and melt out date
        between background and transient periods
        For all simulations or for a given subset, for instance the ones with 'slope'=55
        Note that only the simulations with snow are accounted for,
        otherwise, we would get a huge spike at 0 for all simulations that had no snow and kept it this way.

    Parameters
    ----------
    evol_melt_out : list
        List of the evolution of melt out date
    evol_snow_cover : list
        List of the evolution of snow cover

    Returns
    -------
    fig : figure
        Histogram
    """
    fig, _ = plt.subplots()

    # plots both histograms
    plt.hist(evol_snow_cover, bins=20, alpha=0.75, weights=np.ones_like(evol_snow_cover) / len(evol_snow_cover), label='Snow cover')
    plt.hist(evol_melt_out, bins=20, alpha=0.75, weights=np.ones_like(evol_melt_out) / len(evol_melt_out), label='Melt out date')

    mean_snow_cov = np.mean(evol_snow_cover)
    mean_melt_out = np.mean(evol_melt_out)

    # adds a vertical line denoting the mean values
    plt.axvline(mean_snow_cov, color=colorcycle[0], linestyle='dashed', linewidth=2)
    plt.axvline(mean_melt_out, color=colorcycle[1], linestyle='dashed', linewidth=2)

    plt.annotate(r"$\overline{\Delta}_{\rm snow\ cover}=$%s%s [days]" % (("+" if mean_snow_cov > 0 else ""), float(f"{mean_snow_cov:.2f}")),
                 (0.12,0.5), xycoords='figure fraction',
                 fontsize=12, horizontalalignment='left', verticalalignment='top', color=colorcycle[0])
    plt.annotate(r"$\overline{\Delta}_{\rm mod}=$%s%s [days]" % (("+" if mean_melt_out > 0 else ""), float(f"{mean_melt_out:.2f}")),
                 (0.12,0.45), xycoords='figure fraction',
                 fontsize=12, horizontalalignment='left', verticalalignment='top', color=colorcycle[1])

    plt.xlabel('Evolution [days]')
    plt.ylabel('Frequency')

    # Show the graph
    plt.legend(loc='upper left')
    plt.show()
    plt.close()

    return fig

def plot_evolution_snow_cover_melt_out_from_inputs(site, path_pickle, variable=None, value=None):
    """ Function returns a histogram of the evolution of snow cover (in days) and melt out date
        between background and transient periods
        For all simulations or for a given subset, for instance the ones with 'slope'=55
        Note that only the simulations with snow are accounted for,
        otherwise, we would get a huge spike at 0 for all simulations that had no snow and kept it this way.

    Parameters
    ----------
    evol_melt_out : list
        List of the evolution of melt out date
    evol_snow_cover : list
        List of the evolution of snow cover

    Returns
    -------
    fig : figure
        Histogram
    """

    evol_melt_out, evol_snow_cover = data_evolution_snow_cover_melt_out(site, path_pickle, variable, value)
    fig = plot_evolution_snow_cover_melt_out(evol_melt_out, evol_snow_cover)

    return fig



# This Python file uses the following encoding: utf-8
# This module defines the constants needed for the package

colorcycle = ['#1f77b4', '#ff7f0e', '#2ca02c', '#d62728', '#9467bd',
              '#8c564b', '#e377c2', '#7f7f7f', '#bcbd22', '#17becf']

units = {'GST': '°C', 'Air temperature': '°C',
         'Precipitation': 'mm/day', 'Water production': 'mm/day',
         'SWE': 'mm', 'Snow depth': 'mm',
         'SW': 'W m-2', 'SW direct': 'W m-2', 'SW diffuse': 'W m-2',
         'SW up': 'W m-2', 'SW down': 'W m-2', 'SW net': 'W m-2', 'LW net': 'W m-2'}



"""This module creates statistical analysis based on quantiles"""

#pylint: disable=line-too-long
#pylint: disable=trailing-whitespace
#pylint: disable=invalid-name

import pandas as pd
import numpy as np
import matplotlib
import matplotlib.pyplot as plt
import seaborn as sn

from SuPerSim.pickling import load_all_pickles
from SuPerSim.topoheatmap import table_background_evolution_mean_GST_aspect_slope
from SuPerSim.constants import colorcycle

def coordinates_percentile_cdf(data_sorted, proba_bins, percentile):
    """ Function returns coordinates of the point corresponing to the percentile of the 
        Cumulated Distribution Dunction (CDF)
        
    Parameters
    ----------
    data_sorted : numpy.ndarray
        Sorted data
    proba_bins : numpy.ndarray
        list of sorted probability bins, equally spaced points between 0 and 1 with number of bins equals number of data points
    percentile : int
        Percentile of the CDF

    Returns
    -------
    low_point : list
        Coordinates of the point corresponding to the given percentile on the CDF, in the form: [data, threshold]
    """

    threshold = percentile/100
    k=0
    while proba_bins[k] < threshold:
        k += 1
    if proba_bins[k] == threshold:
        low_point = [data_sorted[k], proba_bins[k]]
    low_point = [(threshold - proba_bins[k-1])/(proba_bins[k] - proba_bins[k-1])*(data_sorted[k] - data_sorted[k-1]) + data_sorted[k-1], threshold]

    return low_point

def data_cdf_GST(site, path_pickle):
    """ Function returns data to plot a Cumulated Distribution Dunction (CDF)
    
    Parameters
    ----------
    site : str
        Location of the event, e.g. 'Joffre' or 'Fingerpost'
    path_pickle : str
        String path to the location of the folder where the pickles are saved

    Returns
    -------
    data : dict
        List of sorted background, transient and evolution of GST (gound surface temperature)
        and SO (surface offset)
    """

    pkl = load_all_pickles(site, path_pickle)
    df_stats = pkl['df_stats']

    # diff_warming stands for differential warming and is the tevolution of the surface offset (SO)
    # between the background and transient periods
    data = {'bkg_grd_temp': np.sort(df_stats['bkg_grd_temp']),
            'trans_grd_temp': np.sort(df_stats['trans_grd_temp']),
            'evol_grd_temp': np.sort(df_stats['evol_grd_temp']),
            'bkg_SO': np.sort(df_stats['bkg_SO']),
            'trans_SO': np.sort(df_stats['trans_SO']),
            'diff_warming': np.sort(df_stats['diff_warming'])}
    
    return data

def plot_cdf_GST(data):
    """ Function plots the Cumulated Distribution Dunction (CDF) of background, transient, and evolution of mean GST and SO
    
    Parameters
    ----------
    data : dict
        List of sorted background, transient and evolution of GST (gound surface temperature)
        and SO (surface offset)

    Returns
    -------
    2 plots: left panel shows the CDF of background, transient, and evolution of mean GST
             right panel shows the CDF of background, transient, and evolution of mean SO
    """
    
    # sort the data:
    data_bkg_sorted = data['bkg_grd_temp']
    data_trans_sorted = data['trans_grd_temp']
    data_evol_sorted = data['evol_grd_temp']

    data_bkg_SO_sorted = data['bkg_SO']
    data_trans_SO_sorted = data['trans_SO']
    data_diff_warming_sorted = data['diff_warming']


    # calculate the proportional values of samples
    p = np.linspace(0, 1, len(data_bkg_sorted))

    point = [[[] for _ in range(3)] for _ in range(4)]

    list_data = [data_bkg_sorted, data_trans_sorted, data_bkg_SO_sorted, data_trans_SO_sorted]
    list_quant = [10,50,90]

    for indx_i, i in enumerate(list_data):
        for indx_j, j in enumerate(list_quant):
            point[indx_i][indx_j] = coordinates_percentile_cdf(i, p, j)

    # plot the sorted data:
    fig, (ax1, ax2) = plt.subplots(1, 2, sharey=True)

    ax1.plot(data_bkg_sorted, p, label='Background', color=colorcycle[0], linewidth=2)
    ax1.plot(data_trans_sorted, p, label='Transient', color=colorcycle[1], linewidth=2)
    ax1.plot(data_evol_sorted, p, label='Evolution', color=colorcycle[2], linewidth=2)

    for i in range(2):
        for j in range(3):
            ax1.scatter(point[i][j][0], point[i][j][1], color=colorcycle[i])

    ylim_ax1 = ax1.get_ylim()
    xlim_ax1 = ax1.get_xlim()
    ax1.axvline(x=0, color='grey', linestyle="dashed")


    ax1.hlines([i/100 for i in list_quant], xlim_ax1[0],
               [np.max(i) for i in np.array(point).transpose()[0].transpose()[:2].transpose()],
               color='black', linestyle="dashed", linewidth=0.5)
    for i in range(2):
        ax1.vlines([j[0] for j in point[i]], ylim_ax1[0],
                   [j/100 for j in list_quant], color=colorcycle[i], linestyle="dashed", linewidth=0.5)

    ax1.set_xlim(xlim_ax1)
    ax1.set_ylim(ylim_ax1)

    ax1.set_xlabel('Mean GST [°C]')
    ax1.set_ylabel('Probability')
    ax1.legend()

    ax2.plot(data_bkg_SO_sorted, p, label='Background SO', color=colorcycle[0], linewidth=2)
    ax2.plot(data_trans_SO_sorted, p, label='Transient SO', color=colorcycle[1], linewidth=2)
    ax2.plot(data_diff_warming_sorted, p, label='Evolution SO', color=colorcycle[2], linewidth=2)
    ylim_ax2 = ax2.get_ylim()
    xlim_ax2 = ax2.get_xlim()
    ax2.axvline(x=0, color='grey', linestyle="dashed")

    for i in range(2,4):
        for j in range(3):
            ax2.scatter(point[i][j][0], point[i][j][1], color=colorcycle[i-2])

    ax2.hlines([i/100 for i in list_quant], xlim_ax1[0],
               [np.max(i) for i in np.array(point).transpose()[0].transpose()[2:].transpose()],
               color='black', linestyle="dashed", linewidth=0.5)
    for i in range(2,4):
        ax2.vlines([j[0] for j in point[i]], ylim_ax1[0],
                   [j/100 for j in list_quant], color=colorcycle[i-2], linestyle="dashed", linewidth=0.5)

    ax2.set_xlim(xlim_ax2)
    ax2.set_ylim(ylim_ax2)

    ax2.set_xlabel('Mean SO [°C]')

    # Show the graph
    plt.show()
    plt.close()

    return fig

def plot_cdf_GST_from_inputs(site, path_pickle):
    """ Function plots the Cumulated Distribution Dunction (CDF) of background, transient, and evolution of mean GST and SO
    
    Parameters
    ----------
    site : str
        Location of the event, e.g. 'Joffre' or 'Fingerpost'
    path_pickle : str
        String path to the location of the folder where the pickles are saved

    Returns
    -------
    2 plots: left panel shows the CDF of background, transient, and evolution of mean GST
             right panel shows the CDF of background, transient, and evolution of mean SO
    """

    data = data_cdf_GST(site, path_pickle)
    fig = plot_cdf_GST(data)

    return fig

def panda_percentile_GST(site, path_pickle):
    """ Function returns a dataframe of 10th, 25th, 50th, 75th, and 90th percentile in background and transient GST, and the difference
    
    Parameters
    ----------
    site : str
        Location of the event, e.g. 'Joffre' or 'Fingerpost'
    path_pickle : str
        String path to the location of the folder where the pickles are saved

    Returns
    -------
    pd_data : pandas.core.frame.DataFrame
        Panda dataframe of 10th, 25th, 50th, 75th, and 90th percentile in background and transient GST, and the difference
    """

    table_all = table_background_evolution_mean_GST_aspect_slope(site, path_pickle)

    # create some randomly distributed data:
    data_bkg = [k for i in table_all['bkg'].values() for j in i.columns for k in i[j]]
    data_evol = [k for i in table_all['evol'].values() for j in i.columns for k in i[j]]
    data_trans = [data_bkg[i]+data_evol[i] for i in range(len(data_bkg))]

    # sort the data:
    data_bkg_sorted = np.sort(data_bkg)
    data_trans_sorted = np.sort(data_trans)

    x = [data_bkg_sorted, data_trans_sorted]
    proba_bins = np.linspace(0, 1, len(data_bkg))
    percentiles = [10, 25, 50, 75, 90]
    table_points = [[coordinates_percentile_cdf(data, proba_bins, p)[0] for data in x] for p in percentiles]

    percentiles.append('Mean')
    table_points.append([np.mean(data_bkg), np.mean(data_trans)])

    pd_data = pd.DataFrame(table_points, index=percentiles, columns=list(['Background', 'Transient']))
    pd_data['Difference'] = pd_data['Transient'] - pd_data['Background']

    pd_data['Background'] = pd.Categorical(pd_data['Background'], np.sort(pd_data['Background']))
    pd_data = pd_data.sort_values('Background')

    return pd_data

def plot_heatmap_percentile_GST(pd_data):
    """ Function returns a heatmap of 10th, 25th, 50th, 75th, and 90th percentile in background and transient GST, and the difference
    
    Parameters
    ----------
    pd_data : pandas.core.frame.DataFrame
        Panda dataframe of 10th, 25th, 50th, 75th, and 90th percentile in background and transient GST, and the difference

    Returns
    -------
    fig : Figure
        Heatmap of 10th, 25th, 50th, 75th, and 90th percentile in background and transient GST, and the difference
    """

    fig, _ = plt.subplots()

    sn.heatmap(data=pd_data, annot=True, fmt=".2f", cmap='seismic',
                cbar=True, yticklabels=True, xticklabels=True, cbar_kws={'label': 'Mean GST [°C]'},
                norm=matplotlib.colors.TwoSlopeNorm(vmin=np.min([np.min(pd_data),-0.05]), vcenter=0, vmax=np.max([np.max(pd_data),0.05])))
    plt.ylabel('Percentile')

    # Show the graph
    plt.show()
    plt.close()

    return fig

def plot_heatmap_percentile_GST_from_inputs(site, path_pickle):
    """ Function returns a heatmap of 10th, 25th, 50th, 75th, and 90th percentile in background and transient GST, and the difference
    
    Parameters
    ----------
    pd_data : pandas.core.frame.DataFrame
        Panda dataframe of 10th, 25th, 50th, 75th, and 90th percentile in background and transient GST, and the difference
        
    Returns
    -------
    fig : Figure
        Heatmap of 10th, 25th, 50th, 75th, and 90th percentile in background and transient GST, and the difference
    """

    pd_data = panda_percentile_GST(site, path_pickle)
    fig = plot_heatmap_percentile_GST(pd_data)

    return fig



"""This module creates a statistical model for ground temperatures vs topography"""

#pylint: disable=line-too-long
#pylint: disable=trailing-whitespace
#pylint: disable=invalid-name

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import scipy.optimize as opt

from SuPerSim.pickling import load_all_pickles
from SuPerSim.constants import colorcycle

def stat_model_aspect_slope_alt(X, offset, c_alt, d_alt, c_asp, c_slope):
    """ Function returns the value of the statistical model 
    
    Parameters
    ----------
    X : list
        List of aspects and altitudes
    offset, c_asp, d_asp, c_slope, d_slope, c_alt : floats
        coefficients of the model

    Returns
    -------
    model_value : float
        Value output of the model given the input
    """

    # This is the statistical model we are trying to fit to the data.
    # unpack the variables in X
    aspect, slope, altitude = X

    model_value = (offset
            + c_alt * altitude
            + c_asp * (altitude - d_alt) * np.cos(aspect * 2 * np.pi / 360) 
            + c_slope * slope)
    
    return model_value 

def data_evol_GST(site, path_pickle, all_data=True, diff_forcings=True):
    """ Function returns the value of the model for the background GST
    
    Parameters
    ----------
    site : str
        Location of the event, e.g. 'Joffre' or 'Fingerpost'
    path_pickle : str
        String path to the location of the folder where the pickles are saved
    all_data : bool, optional
        If True, considers all data at once
    diff_forcings : bool, optional
        If True, separates data by 'forcing'

    Returns
    -------
    data_set : list of pandas.core.frame.DataFrame
        list of dataframes with columns: ['bkg_grd_temp', 'aspect', 'slope', 'altitude', 'forcing']
        one dataframe for all if all_data=True plus one dataframe per forcing if diff_forcings=True
    """

    pkl = load_all_pickles(site, path_pickle)
    df_stats = pkl['df_stats']
    df_stats = df_stats[['bkg_grd_temp', 'aspect', 'slope', 'altitude', 'forcing']]

    forcings = np.unique(df_stats['forcing'])
    data_set = []
    if all_data:
        data_set.append(df_stats)
    else: 
        pass
    if diff_forcings:
        for i in forcings:
            data_set.append(df_stats[df_stats['forcing']==i])
    else:
        pass

    return data_set

def fit_stat_model_GST(data_set, all_data=True):
    """ Function returns the value of the statistical model 
    
    Parameters
    ----------
    data_set : list of pandas.core.frame.DataFrame
        list of dataframes with columns: ['bkg_grd_temp', 'aspect', 'slope', 'altitude', 'forcing']
        one dataframe for all if all_data=True plus one dataframe per forcing if diff_forcings=True
    all_data : bool, optional
        If True, considers all data at once

    Returns
    -------
    fig : Figure
        Parity plot (predicted vs actual) background GST
    xdata : list
        List of xdata (actual) grouped by forcing if all_data=False
    ydata : list
        List of ydata (predicted) grouped by forcing if all_data=False
    optimizedParameters : list
        List of optimized model parameters grouped by forcing if all_data=False
    pcov : list
        List of covariances grouped by forcing if all_data=False
    corr_matrix : list
        List of correlation matrices grouped by forcing if all_data=False
    R_sq : list
        List of R^2 grouped by forcing if all_data=False
    """

    input_var = [np.array([i['aspect'], i['slope'], i['altitude']]) for i in data_set]
    # all the measured differential warmings (from valid simulations) are in xdata
    xdata = [np.array(i['bkg_grd_temp']) for i in data_set]

    forcings = []
    if all_data:
        forcings.append('all')
    for i in data_set[(1 if all_data else 0):]:
        forcings.append(list(np.unique(i['forcing']))[0])
    
    # The actual curve fitting happens here
    ydata = []
    R_sq = []
    optimizedParameters = []
    pcov = []
    corr_matrix = []
    bounds=((-50, -np.inf, -np.inf, -np.inf, -np.inf), (50, np.inf, np.inf, np.inf, np.inf))
    p0 = (0,0,1000,0,0)

    fig = plt.figure(figsize=(6,6))

    for i,in_var in enumerate(input_var):
        optimizedParameters.append(opt.curve_fit(stat_model_aspect_slope_alt, in_var, xdata[i], bounds=bounds, p0=p0)[0])
        pcov.append(opt.curve_fit(stat_model_aspect_slope_alt, in_var, xdata[i], bounds=bounds, p0=p0)[0])

        # this represents the fitted values, hence they are the statistically-modelled values 
        # of differential warming: we call them ydata
        ydata.append(stat_model_aspect_slope_alt(in_var, *optimizedParameters[i]))

        # R^2 from numpy package, to check!
        corr_matrix.append(np.corrcoef(xdata[i], ydata[i]))
        corr = corr_matrix[i][0,1]
        R_sq.append(corr**2)

        plt.scatter(xdata[i], ydata[i], marker=("D" if (all_data and i==0) else "o"),
                    s=20, label=('all data' if (all_data and i==0) else forcings[len(forcings)-len(data_set)+i]) )
        
    # plot the y=x diagonal
    # start by setting the bounds

    lim_up = float(f"{max(np.max([np.max(i) for i in xdata]), np.max([np.max(i) for i in ydata])):.2g}")
    lim_down = float(f"{min(np.min([np.min(i) for i in xdata]), np.min([np.min(i) for i in ydata])):.2g}")
    x = np.arange(lim_down, lim_up, 0.01)
    plt.plot(x, x, color=colorcycle[len(data_set)], linestyle='dashed', label = 'y=x', linewidth=2)
    plt.legend(loc='upper right')

    margin = 0.1
    plt.ylim(ymin= lim_down - margin, ymax= lim_up + margin)
    plt.xlim(xmin= lim_down - margin, xmax= lim_up + margin)

    plt.xlabel(r'Numerically-simulated background GST $\overline{T_{\rm GST}^{\rm bkg}}_{(NS)}$ [°C]')
    plt.ylabel(r'Statistically-modelled background GST $\overline{T_{\rm GST}^{\rm bkg}}_{(SM)}$ [°C]')
    for i,r in enumerate(R_sq):
        plt.figtext(.7, .3 - i/30, f"$R^2$ = {r:.2f}",
                    c=colorcycle[i])

    # Show the graph
    plt.legend()
    plt.show()
    plt.close()

    list_coef = ['offset', 'c_alt', 'd_alt', 'c_asp', 'c_slope']
    pd_coef = pd.DataFrame(list_coef, columns=['Coefficient'])
    pd_coef = pd.concat([pd_coef, pd.DataFrame((np.array([list(i) for i in optimizedParameters]).transpose()), columns=forcings)], axis=1)
    print('The model is given by the function: ')
    print('(offset + c_alt * altitude + c_asp * (altitude - d_alt) * np.cos(aspect * 2 * np.pi / 360) + c_slope * slope)')
    print('And the model coefficients are given by: ')
    print(pd_coef)

    return fig, xdata, ydata, optimizedParameters, pcov, corr_matrix, R_sq

def fit_stat_model_GST_from_inputs(site, path_pickle, all_data=True, diff_forcings=True):
    """ Function returns the value of the statistical model 
    
    Parameters
    ----------
    site : str
        Location of the event, e.g. 'Joffre' or 'Fingerpost'
    path_pickle : str
        String path to the location of the folder where the pickles are saved
    all_data : bool, optional
        If True, considers all data at once
    diff_forcings : bool, optional
        If True, separates data by 'forcing'

    Returns
    -------
    fig : Figure
        Parity plot (predicted vs actual) background GST
    xdata : list
        List of xdata (actual) grouped by forcing if all_data=False
    ydata : list
        List of ydata (predicted) grouped by forcing if all_data=False
    optimizedParameters : list
        List of optimized model parameters grouped by forcing if all_data=False
    pcov : list
        List of covariances grouped by forcing if all_data=False
    corr_matrix : list
        List of correlation matrices grouped by forcing if all_data=False
    R_sq : list
        List of R^2 grouped by forcing if all_data=False
    """

    data_set = data_evol_GST(site, path_pickle, all_data, diff_forcings)
    fig, xdata, ydata, optimizedParameters, pcov, corr_matrix, R_sq = fit_stat_model_GST(data_set, all_data)

    return fig, xdata, ydata, optimizedParameters, pcov, corr_matrix, R_sq



"""This module automates the plotting of summary statistics for timeseries"""

#pylint: disable=line-too-long
#pylint: disable=trailing-whitespace
#pylint: disable=invalid-name

import numpy as np

from SuPerSim.open import open_air_nc, open_ground_nc, open_snow_nc, open_swe_nc, open_SW_direct_nc, open_SW_diffuse_nc, open_SW_up_nc, open_SW_down_nc, open_SW_net_nc, open_LW_net_nc
from SuPerSim.mytime import list_tokens_year
from SuPerSim.pickling import load_all_pickles
from SuPerSim.weights import assign_weight_sim, plot_hist_stat_weights_from_input, plot_hist_valid_sim_all_variables_from_input
from SuPerSim.runningstats import mean_all_altitudes, mean_all_reanalyses, assign_tot_water_prod, plot_aggregating_distance_temp_all_from_input
from SuPerSim.topoheatmap import plot_table_mean_GST_aspect_slope_single_altitude_from_inputs, plot_table_mean_GST_aspect_slope_all_altitudes_from_inputs, plot_table_mean_GST_aspect_slope_all_altitudes_polar_from_inputs, plot_permafrost_all_altitudes_polar_from_inputs
from SuPerSim.model import fit_stat_model_GST_from_inputs
from SuPerSim.percentiles import plot_cdf_GST_from_inputs, plot_heatmap_percentile_GST_from_inputs
from SuPerSim.yearlystats import plot_box_yearly_stat_from_inputs, plot_yearly_quantiles_atmospheric_from_inputs, plot_yearly_quantiles_sim_from_inputs, plot_yearly_quantiles_side_by_side_sim_from_inputs
from SuPerSim.seasonal import plot_sanity_one_year_quantiles_two_periods_from_inputs, plot_sanity_two_variables_one_year_quantiles_from_inputs, plot_sanity_two_variables_two_sites_one_year_quantiles_side_by_side_from_inputs
from SuPerSim.evolution import plot_evolution_snow_cover_melt_out_from_inputs, plot_GST_bkg_vs_evol_quantile_bins_fit_single_site_from_inputs, plot_mean_bkg_GST_vs_evolution_from_inputs, plot_GST_bkg_vs_evol_quantile_bins_fit_two_sites_from_input
from SuPerSim.horizon import plot_visible_skymap_from_horizon_file

def plot_all(site,
             path_forcing_list, path_ground, path_snow, path_swe, path_thaw_depth, path_pickle,
             year_bkg_end, year_trans_end, path_horizon=None, no_weight=True, show_glaciers=True,
             individual_heatmap=False, polar_plots=False, parity_plot=False):
    """ Function returns a series of summary plots for a given site.
    
    Parameters
    ----------
    site : str
        Location of the event, e.g. 'Joffre' or 'Fingerpost'
    path_forcing_list : list of str
        List of paths to the .nc file where the atmospheric forcing data for the given reanalysis is stored
    path_ground : str
        String path to the location of the ground output file from GTPEM (.nc)
    path_snow : str
        Path to the .nc file where the aggregated snow simulations are stored
    path_swe : str
        Path to the .nc file where the aggregated SWE simulations are stored
    path_thaw_depth : str
        String path to the location of the thaw depth output file from GTPEM (.nc)
    path_pickle : str
        String path to the location of the folder where the pickles are saved
    year_bkg_end : int
        Background period is BEFORE the start of the year corresponding to the variable, i.e. all time stamps before Jan 1st year_bkg_end
    year_trans_end : int
        Same for transient period
    path_horizon : str, optional
        If a path to a .csv horizon file is given, a horizon plot is produced, if None, nothing happens.
    no_weight : bool, optional
        If True, all simulations have the same weight, otherwise the weight is computed as a function of altitude, aspect, and slope
    show_glaciers : bool, optional
        If True, shows the glacier simulations with a 0 weight, if False, those are ignored.
    individual_heatmap : bool, optional
        Show or not heatmaps for unique altitude
    polar_plots : bool, optional
        Show or not polar plots
    parity_plot : bool, optional
        Show or not parity plot

    Returns
    -------
    Plots
    """  

    #####################################################################################
    # OPEN THE VARIOUS FILES
    #####################################################################################

    pkl = load_all_pickles(site, path_pickle)
    # df = pkl['df']
    list_valid_sim = pkl['list_valid_sim']
    df_stats = pkl['df_stats']
    rockfall_values = pkl['rockfall_values']

    #####################################################################################
    # PLOTS
    #####################################################################################

    # assign a subjective weight to all simulations
    pd_weight, _ = assign_weight_sim(site, path_pickle, no_weight)

    list_vars = ['time_air', 'temp_air', 'SW_flux', 'SW_direct_flux', 'SW_diffuse_flux', 'precipitation']
    list_series = [open_air_nc(i) for i in path_forcing_list]
    list_series_b = [[list_series[j][i] for j in range(len(list_series))] for i in range(len(list_series[0]))]
    air_all_dict = dict(zip(list_vars, list_series_b))

    time_air_all = air_all_dict['time_air']
    temp_air_all = air_all_dict['temp_air']
    precipitation_all = air_all_dict['precipitation']

    _, time_ground, temp_ground = open_ground_nc(path_ground)
    _, snow_height = open_snow_nc(path_snow)
    _, swe = open_swe_nc(path_swe)

    _, time_bkg_ground, time_trans_ground, _ = list_tokens_year(time_ground, year_bkg_end, year_trans_end)

    # weighted mean GST
    temp_ground_mean = list(np.average([temp_ground[i,:,0] for i in list(pd_weight.index.values)], axis=0, weights=pd_weight.loc[:, 'weight']))
    
    # Mean air temperature over all reanalyses and altitudes
    mean_air_temp = mean_all_reanalyses(time_air_all,
                                        [mean_all_altitudes(i, site, path_pickle, no_weight) for i in temp_air_all],
                                        year_bkg_end, year_trans_end)
    # mean_air_temp = mean_all_reanalyses(time_air_all, [i[:,alt_index] for i in temp_air_all], year_bkg_end, year_trans_end)

    # finally we get the total water production, averaged over all reanalyses
    tot_water_prod, _, mean_prec = assign_tot_water_prod(path_forcing_list, path_ground, path_swe, path_pickle, year_bkg_end, year_trans_end, site, no_weight)

    alt_list = sorted(set(df_stats['altitude']))
    alt_index = int(np.floor((len(alt_list)-1)/2))
    alt_index_abs = alt_list[alt_index]

    if path_horizon is not None:
        print('\n---------------------------------------------------------------------------------------------\n')
        print('Fisheye view of the sky with the visible portion in blue and the blocked one in black:')
        plot_visible_skymap_from_horizon_file(path_horizon)

    print('\n---------------------------------------------------------------------------------------------\n')

    print('The following plot is a histogram of the distribution of the statistical weights of all simulations:')
    plot_hist_stat_weights_from_input(site, path_pickle, no_weight, show_glaciers)

    print('\n---------------------------------------------------------------------------------------------\n')

    print('The following plot is a histogram of the distribution of glacier simulations wrt to altitude, aspect, slope, and forcing:')
    plot_hist_valid_sim_all_variables_from_input(site, path_thaw_depth, path_pickle)



    print('\n---------------------------------------------------------------------------------------------')
    print('------------------------------------- TEMPORAL ANALYSIS -------------------------------------')
    print('---------------------------------------------------------------------------------------------\n')

    print('\n---------------------------------------------------------------------------------------------\n')
    print('Yearly statistics for air and ground surface temperature, and also precipitation and water production')
    plot_box_yearly_stat_from_inputs('Air temperature', time_air_all[0], mean_air_temp, year_bkg_end, year_trans_end)
    plot_box_yearly_stat_from_inputs('GST', time_ground, temp_ground_mean, year_bkg_end, year_trans_end)
    plot_box_yearly_stat_from_inputs('Precipitation', time_air_all[0], mean_prec, year_bkg_end, year_trans_end)
    plot_box_yearly_stat_from_inputs('Water production', time_ground, tot_water_prod, year_bkg_end, year_trans_end)

    print('\n---------------------------------------------------------------------------------------------\n')
    print('Plot of yearly statistics for atmospheric timeseries. Mean and several quantiles for each year:')
    plot_yearly_quantiles_atmospheric_from_inputs(time_air_all, temp_air_all, 'Air temperature', year_bkg_end, year_trans_end)
    plot_yearly_quantiles_atmospheric_from_inputs(time_air_all, precipitation_all, 'Precipitation', year_bkg_end, year_trans_end)

    print('\n---------------------------------------------------------------------------------------------\n')
    print('Plot of yearly statistics for simulated timeseries. Mean and several quantiles for each year:')
    plot_yearly_quantiles_sim_from_inputs(time_ground, temp_ground, list_valid_sim, 'GST', year_bkg_end, year_trans_end)
    plot_yearly_quantiles_sim_from_inputs(time_ground, snow_height, list_valid_sim, 'Snow depth', year_bkg_end, year_trans_end)
    plot_yearly_quantiles_sim_from_inputs(time_ground, swe, list_valid_sim, 'SWE', year_bkg_end, year_trans_end)

    print('\n---------------------------------------------------------------------------------------------\n')
    print('Plot of 2 timeseries reduced to a 1-year window with mean and 1- and 2-sigma spread:')
    plot_sanity_two_variables_one_year_quantiles_from_inputs(time_ground, [temp_ground, snow_height], [list_valid_sim, list_valid_sim], ['GST', 'Snow depth'])

    print('\n---------------------------------------------------------------------------------------------\n')
    print('Plot of a single timeseries reduced to a 1-year window with mean and 1 and 2-sigma spread, for background and transient periods:')
    plot_sanity_one_year_quantiles_two_periods_from_inputs(time_ground, [temp_ground, temp_ground], [list_valid_sim, list_valid_sim], 'GST', ['Background', 'Transient'], [time_bkg_ground, time_trans_ground])
    plot_sanity_one_year_quantiles_two_periods_from_inputs(time_ground, [snow_height, snow_height], [list_valid_sim, list_valid_sim], 'Snow depth', ['Background', 'Transient'], [time_bkg_ground, time_trans_ground])

    # # print('')
    # This works well but it would be better to smooth the data
    # plot_sanity_one_year_quantiles_two_periods_from_inputs(time_air_all[0], [temp_air_all[0], temp_air_all[0]], [None, None], 'Air temperature', ['Background', 'Transient'], [time_bkg_air, time_trans_air])

    print('Plots of the normalized distance of air and ground temperature, water production, and thaw_depth as a function of time')
    if 'year' in rockfall_values.keys():
        year_rockfall = rockfall_values['year']
        print('\n---------------------------------------------------------------------------------------------\n')
        print('Granularity: week and month side by side')
        plot_aggregating_distance_temp_all_from_input(['Air temperature', 'Water production', 'Ground temperature'],
                                        [time_air_all[0], time_ground, time_ground],
                                        [mean_air_temp, tot_water_prod, temp_ground_mean],
                                        ['week', 'month'], site, path_pickle, year_bkg_end, year_trans_end, year_rockfall, False)
    
    print('\n---------------------------------------------------------------------------------------------\n')
    print('Granularity: year, plotted for all years')
    plot_aggregating_distance_temp_all_from_input(['Air temperature', 'Water production', 'Ground temperature'],
                                        [time_air_all[0], time_ground, time_ground],
                                        [mean_air_temp, tot_water_prod, temp_ground_mean],
                                        ['year'], site, path_pickle, year_bkg_end, year_trans_end, 0, False)



    print('\n---------------------------------------------------------------------------------------------')
    print('------------------------------------- SPATIAL ANALYSIS --------------------------------------')
    print('---------------------------------------------------------------------------------------------\n')




    if individual_heatmap:
        alt_show = rockfall_values['altitude'] if ((rockfall_values['exact_topo']) and (rockfall_values['altitude'] in alt_list)) else alt_index_abs
        print('\n---------------------------------------------------------------------------------------------\n')
        print(f'Heatmap of the background mean GST as a function of aspect and slope at {alt_show} m:')
        plot_table_mean_GST_aspect_slope_single_altitude_from_inputs(site, path_pickle, alt_show, background=True, box=True)
        print('\n---------------------------------------------------------------------------------------------\n')
        print(f'Heatmap of the evolution of the mean GST between the background and the transient periods as a function of aspect and slope at {alt_show} m:')
        plot_table_mean_GST_aspect_slope_single_altitude_from_inputs(site, path_pickle, alt_show, background=False, box=True)

    print('\n---------------------------------------------------------------------------------------------\n')
    print('Heatmap of the background mean GST and its evolution as a function of aspect and slope at all altitudes')
    plot_table_mean_GST_aspect_slope_all_altitudes_from_inputs(site, path_pickle, show_glaciers=True, box=True)


    if polar_plots:
        print('\n---------------------------------------------------------------------------------------------\n')
        print('Polar heatmap of the background mean GST and its evolution as a function of aspect and slope at all altitude')
        plot_table_mean_GST_aspect_slope_all_altitudes_polar_from_inputs(site, path_pickle, box=True)

        print('\n---------------------------------------------------------------------------------------------\n')
        print('Polar plot of the permafrost and glacier spatial distribution as a function of aspect and slope at all altitude')
        plot_permafrost_all_altitudes_polar_from_inputs(site, path_pickle, path_thaw_depth, box=True)



    print('\n---------------------------------------------------------------------------------------------')
    print('-------------------------------------- FURTHER  PLOTS ---------------------------------------')
    print('---------------------------------------------------------------------------------------------\n')



    print('\n---------------------------------------------------------------------------------------------\n')
    print('CDF of background, transient, and evolution GST:')
    plot_cdf_GST_from_inputs(site, path_pickle)

    print('\n---------------------------------------------------------------------------------------------\n')
    print('Heatmap of 10th, 25th, 50th, 75th, and 90th percentile in background and transient GST, and the difference:')
    plot_heatmap_percentile_GST_from_inputs(site, path_pickle)

    print('\n---------------------------------------------------------------------------------------------\n')
    print('Histogram of the evolution of the snow cover (in days) and melt-out date:')
    plot_evolution_snow_cover_melt_out_from_inputs(site, path_pickle)

    print('\n---------------------------------------------------------------------------------------------\n')
    print('Plot of mean GST evolution vs background GST, fit, and binning per 10% quantiles')
    plot_GST_bkg_vs_evol_quantile_bins_fit_single_site_from_inputs(site, path_pickle)

    print('\n---------------------------------------------------------------------------------------------\n')
    print('Scatter plot of mean background GST vs evolution of mean GST between the background and transient period')
    plot_mean_bkg_GST_vs_evolution_from_inputs(site, path_pickle)

    if parity_plot:
        print('\n---------------------------------------------------------------------------------------------\n')
        print('Parity plot (statistically-modeled vs numerically-simulated) of background mean GST:')
        fit_stat_model_GST_from_inputs(site, path_pickle, all_data=False, diff_forcings=True)

    
    print('\n---------------------------------------------------------------------------------------------')
    print('---------------------------------- SUCCESSFULLY COMPLETED -----------------------------------')
    print('---------------------------------------------------------------------------------------------\n')



def plot_camparison_two_sites(list_site, list_label_site,
             list_path_forcing_list, list_path_ground, list_path_snow, list_path_swe,
             list_path_SW_direct, list_path_SW_diffuse, list_path_SW_up,
             list_path_SW_down, list_path_SW_net, list_path_LW_net,
             list_path_pickle, year_bkg_end, year_trans_end):
    """ Function returns a series of comparison of summary plots for two sites.
    
    Parameters
    ----------
    list_site : str
        List of location of the event, e.g. ['Joffre', 'Fingerpost'] or ['North', 'South']
    list_label_site : list of str
        List of label for each site
    list_path_forcing_list : list of str
        List of list of paths to the .nc file where the atmospheric forcing data for the given reanalysis is stored
        E.g. [['path_reanalysis_1_North', .., 'path_reanalysis_n_North'], ['path_reanalysis_1_South', .., 'path_reanalysis_n_South']]
    list_path_ground : list of str
        List of string path to the location of the ground output file from GTPEM (.nc)
    list_path_snow : str
        List of path to the .nc file where the aggregated snow simulations are stored
    list_path_swe : str
        List of path to the .nc file where the aggregated SWE simulations are stored
    list_path_SW_direct : str
        List of path to the .nc file where the aggregated SW direct simulations are stored
    list_path_SW_diffuse : str
        List of path to the .nc file where the aggregated SW diffuse simulations are stored
    list_path_SW_up : str
        List of path to the .nc file where the aggregated SW up simulations are stored
    list_path_SW_down : str
        List of path to the .nc file where the aggregated SW down simulations are stored
    list_path_SW_net : str
        List of path to the .nc file where the aggregated SW net simulations are stored
    list_path_LW_net : str
        List of path to the .nc file where the aggregated LW net simulations are stored
    list_path_pickle : str
        List of path to the location of the folder where the pickles are saved
    year_bkg_end : int
        Background period is BEFORE the start of the year corresponding to the variable, i.e. all time stamps before Jan 1st year_bkg_end
    year_trans_end : int
        Same for transient period

    Returns
    -------
    Plots
    """ 

    [df, list_valid_sim, df_stats, rockfall_values,
    time_air_all, temp_air_all, precipitation_all,
    time_ground, temp_ground, snow_height, swe,
    SW_direct, SW_diffuse, SW_up, SW_down, SW_net, LW_net] = [[[] for _ in range(2)] for _ in range(17)]

    #####################################################################################
    # OPEN THE VARIOUS FILES
    #####################################################################################

    for i in range(2):
        pkl = load_all_pickles(list_site[i], list_path_pickle[i])
        df[i] = pkl['df']
        list_valid_sim[i] = pkl['list_valid_sim']
        df_stats[i] = pkl['df_stats']
        rockfall_values[i] = pkl['rockfall_values']

        list_vars = ['time_air', 'temp_air', 'precipitation']
        list_series = [open_air_nc(j) for j in list_path_forcing_list[i]]
        list_series_b = [[list_series[j][k] for j in range(len(list_series))] for k in range(len(list_series[0]))]
        air_all_dict = dict(zip(list_vars, list_series_b))

        time_air_all[i] = air_all_dict['time_air']
        temp_air_all[i] = air_all_dict['temp_air']
        precipitation_all[i] = air_all_dict['precipitation']

        _, time_ground[i], temp_ground[i] = open_ground_nc(list_path_ground[i])
        _, snow_height[i] = open_snow_nc(list_path_snow[i])
        _, swe[i] = open_swe_nc(list_path_swe[i])
        _, SW_direct[i] = open_SW_direct_nc(list_path_SW_direct[i])
        _, SW_diffuse[i] = open_SW_diffuse_nc(list_path_SW_diffuse[i])
        _, SW_up[i] = open_SW_up_nc(list_path_SW_up[i])
        _, SW_down[i] = open_SW_down_nc(list_path_SW_down[i])
        _, SW_net[i] = open_SW_net_nc(list_path_SW_net[i])
        _, LW_net[i] = open_LW_net_nc(list_path_LW_net[i])

    print('\n---------------------------------------------------------------------------------------------\n')

    print('Series of plots comparing both sites.')

    print('\n---------------------------------------------------------------------------------------------\n')

    print('Plot of a single timeseries reduced to a 1-year window with mean and 1 and 2-sigma spread, for both sites:')
    plot_sanity_two_variables_one_year_quantiles_from_inputs(time_ground[0], [temp_ground[0], temp_ground[1]], [list_valid_sim[0], list_valid_sim[1]], ['GST'], list_label_site)
    plot_sanity_two_variables_one_year_quantiles_from_inputs(time_ground[0], [SW_direct[0], SW_direct[1]], [list_valid_sim[0], list_valid_sim[1]], ['SW direct'], list_label_site)
    plot_sanity_two_variables_one_year_quantiles_from_inputs(time_ground[0], [SW_diffuse[0], SW_diffuse[1]], [list_valid_sim[0], list_valid_sim[1]], ['SW diffuse'], list_label_site)
    plot_sanity_two_variables_one_year_quantiles_from_inputs(time_ground[0], [SW_down[0], SW_down[1]], [list_valid_sim[0], list_valid_sim[1]], ['SW up'], list_label_site)
    plot_sanity_two_variables_one_year_quantiles_from_inputs(time_ground[0], [SW_up[0], SW_up[1]], [list_valid_sim[0], list_valid_sim[1]], ['SW down'], list_label_site)
    plot_sanity_two_variables_one_year_quantiles_from_inputs(time_ground[0], [SW_net[0], SW_net[1]], [list_valid_sim[0], list_valid_sim[1]], ['SW net'], list_label_site)
    plot_sanity_two_variables_one_year_quantiles_from_inputs(time_ground[0], [LW_net[0], LW_net[1]], [list_valid_sim[0], list_valid_sim[1]], ['LW net'], list_label_site)

    print('\n---------------------------------------------------------------------------------------------\n')

    print('Plot of seasonal statistics for GST and SW net for both sites side by side:')
    plot_sanity_two_variables_two_sites_one_year_quantiles_side_by_side_from_inputs(time_ground[0], [[temp_ground[0], temp_ground[1]], [SW_net[0], SW_net[1]]], [list_valid_sim[0], list_valid_sim[1]], ['GST', 'SW net'], list_label_site)
    
    print('\n---------------------------------------------------------------------------------------------\n')

    print('Plot of yearly, background, and transient statistics for GST for both sites side by side:')
    plot_yearly_quantiles_side_by_side_sim_from_inputs(time_ground[0], [temp_ground[0], temp_ground[1]], [list_valid_sim[0], list_valid_sim[1]], 'GST', list_label_site, year_bkg_end, year_trans_end)
    
    print('\n---------------------------------------------------------------------------------------------\n')

    print('Plot of mean GST evolution vs background GST, fit, and binning per 10% quantiles for both sites:')
    plot_GST_bkg_vs_evol_quantile_bins_fit_two_sites_from_input(list_site, list_path_pickle, list_label_site)

    
    print('\n---------------------------------------------------------------------------------------------')
    print('---------------------------------- SUCCESSFULLY COMPLETED -----------------------------------')
    print('---------------------------------------------------------------------------------------------\n')



"""This module creates a horizon plot (fisheye sky view)"""

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt

def read_horizon_to_polar(path_horizon):
    """ Function read a .csv horizon file with columns 'azi' (azimuth) and 'hang' (horizon angle) and formats them.
    
    Parameters
    ----------
    path_horizon : str
        Path to the .csv horizon file

    Returns
    -------
    theta : numpy.ndarray
        azimuth angle (clockwise from North) in degrees increasing from 0 to 2pi
    zenith_angle : numpy.ndarray
        horizon angle in degrees (from the zenith) at corresponding theta (azimuth) direction
        from 0 to 90
        0 corresponds to zenith (vertical wall), 90 corresponds to flat horizon
    """

    hor_file = pd.read_csv(path_horizon, usecols=['azi', 'hang'])
    theta_pre = hor_file['azi']
    theta = np.array([i/360*2*np.pi for i in theta_pre])
    zenith_angle_pre = hor_file['hang']
    zenith_angle = np.array([90-i for i in zenith_angle_pre])

    return theta, zenith_angle

def plot_visible_skymap(theta, zenith_angle):
    """ Function returns a fisheye view of the sky with the visible portion in blue and the blocked one in black.
    
    Parameters
    ----------
    theta : numpy.ndarray
        azimuth angle (clockwise from North) in degrees increasing from 0 to 2pi
    zenith_angle : numpy.ndarray
        horizon angle in degrees (from the zenith) at corresponding theta (azimuth) direction
        from 0 to 90
        0 corresponds to zenith (vertical wall), 90 corresponds to flat horizon

    Returns
    -------
    fig : figure
        Plot of the sky view from the location
    """

    # Creating the polar scatter plot
    fig = plt.figure(figsize=(6,6))
    ax = plt.subplot(111, polar=True)
    ax.set_theta_direction(-1)
    ax.set_theta_offset(np.pi/2)
    ax.set_ylim(0, 90)
    for i,j in enumerate(zenith_angle):
        ax.fill_between(np.linspace(theta[i], (0 if i==len(zenith_angle)-1 else theta[i+1]), 2), 0, j, color='deepskyblue', alpha=0.5)
        ax.fill_between(np.linspace(theta[i], (0 if i==len(zenith_angle)-1 else theta[i+1]), 2), j, 90, color='black', alpha=1)
    ax.scatter(theta, zenith_angle, color='blue', s=10, alpha=0.75)
    # plt.title('Scatter Plot on Polar Axis', fontsize=15)

    plt.show()
    plt.close()

    return fig

def plot_visible_skymap_from_horizon_file(path_horizon):
    """ Function returns a fisheye view of the sky with the visible portion in blue and the blocked one in black
        from a .csv horizon file
    
    Parameters
    ----------
    path_horizon : str
        Path to the .csv horizon file

    Returns
    -------
    fig : figure
        Plot of the sky view from the location
    """

    theta, zenith_angle = read_horizon_to_polar(path_horizon)
    fig = plot_visible_skymap(theta, zenith_angle)

    return fig



""" Imports functions that could be re-used by a user in a script """
#pylint: disable=line-too-long

# creating and loading pickles
from SuPerSim.pickling import load_all_pickles, get_all_stats

# producing all plots for either 1 site or for comparing 2 sites
from SuPerSim.functions_summary import plot_camparison_two_sites, plot_all

# all the individual plotting functions
# plotting directly from the inputs
from SuPerSim.horizon import plot_visible_skymap_from_horizon_file
from SuPerSim.weights import plot_hist_stat_weights_from_input, plot_hist_valid_sim_all_variables_from_input
from SuPerSim.runningstats import plot_aggregating_distance_temp_all_from_input
from SuPerSim.yearlystats import plot_box_yearly_stat_from_inputs
from SuPerSim.topoheatmap import plot_table_mean_GST_aspect_slope_single_altitude_from_inputs, plot_table_mean_GST_aspect_slope_all_altitudes_from_inputs, plot_table_mean_GST_aspect_slope_all_altitudes_polar_from_inputs, plot_permafrost_all_altitudes_polar_from_inputs
from SuPerSim.percentiles import plot_cdf_GST_from_inputs, plot_heatmap_percentile_GST_from_inputs
from SuPerSim.evolution import plot_GST_bkg_vs_evol_quantile_bins_fit_single_site_from_inputs, plot_mean_bkg_GST_vs_evolution_from_inputs, plot_evolution_snow_cover_melt_out_from_inputs, plot_GST_bkg_vs_evol_quantile_bins_fit_two_sites_from_input
from SuPerSim.model import fit_stat_model_GST_from_inputs
from SuPerSim.yearlystats import plot_yearly_quantiles_atmospheric_from_inputs, plot_yearly_quantiles_sim_from_inputs, plot_yearly_quantiles_side_by_side_sim_from_inputs, plot_yearly_quantiles_side_by_side_atmospheric_from_inputs
from SuPerSim.seasonal import plot_sanity_two_variables_one_year_quantiles_from_inputs, plot_sanity_one_year_quantiles_two_periods_from_inputs, plot_sanity_two_variables_two_sites_one_year_quantiles_side_by_side_from_inputs

# plotting from formatted data that a user can produce
from SuPerSim.horizon import plot_visible_skymap
from SuPerSim.weights import plot_hist_stat_weights, plot_hist_valid_sim_all_variables
from SuPerSim.runningstats import plot_aggregating_distance_temp_all
from SuPerSim.yearlystats import plot_box_yearly_stat
from SuPerSim.topoheatmap import plot_table_mean_GST_aspect_slope_single_altitude, plot_table_mean_GST_aspect_slope_all_altitudes, plot_table_mean_GST_aspect_slope_all_altitudes_polar, plot_permafrost_all_altitudes_polar
from SuPerSim.percentiles import plot_cdf_GST, plot_heatmap_percentile_GST
from SuPerSim.evolution import plot_GST_bkg_vs_evol_quantile_bins_fit_single_site, plot_mean_bkg_GST_vs_evolution, plot_evolution_snow_cover_melt_out, plot_GST_bkg_vs_evol_quantile_bins_fit_two_sites
from SuPerSim.model import fit_stat_model_GST
from SuPerSim.yearlystats import plot_yearly_quantiles, plot_yearly_quantiles_side_by_side
from SuPerSim.seasonal import plot_sanity_two_variables_one_year_quantiles, plot_sanity_one_year_quantiles_two_periods, plot_sanity_two_variables_two_sites_one_year_quantiles_side_by_side
